@@ -605,21 +605,18 @@ namespace Utils {
 //
 // TreeItem
 //
-TreeItem::TreeItem()
-    : m_parent(0), m_model(0)
-{
-}
+TreeItem::TreeItem() = default;
 
 TreeItem::~TreeItem()
 {
-    QTC_CHECK(m_parent == 0);
-    QTC_CHECK(m_model == 0);
+    QTC_CHECK(m_parent == nullptr);
+    QTC_CHECK(m_model == nullptr);
     removeChildren();
 }
 
 TreeItem *TreeItem::childAt(int pos) const
 {
-    QTC_ASSERT(pos >= 0, return 0);
+    QTC_ASSERT(pos >= 0, return nullptr);
     return pos < childCount() ? *(begin() + pos) : nullptr;
 }
 
@@ -837,7 +834,7 @@ void TreeItem::forChildrenAtLevel(int level, const std::function<void(TreeItem *
 
 TreeItem *TreeItem::findChildAtLevel(int level, const std::function<bool(TreeItem *)> &pred) const
 {
-    QTC_ASSERT(level > 0, return 0);
+    QTC_ASSERT(level > 0, return nullptr);
     if (level == 1) {
         for (TreeItem *item : *this)
             if (pred(item))
@@ -848,7 +845,7 @@ TreeItem *TreeItem::findChildAtLevel(int level, const std::function<bool(TreeIte
                 return found;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 TreeItem *TreeItem::findAnyChild(const std::function<bool(TreeItem *)> &pred) const
@@ -859,17 +856,17 @@ TreeItem *TreeItem::findAnyChild(const std::function<bool(TreeItem *)> &pred) co
         if (TreeItem *found = item->findAnyChild(pred))
             return found;
     }
-    return 0;
+    return nullptr;
 }
 
 TreeItem *TreeItem::reverseFindAnyChild(const std::function<bool (TreeItem *)> &pred) const
 {
     auto end = m_children.rend();
     for (auto it = m_children.rbegin(); it != end; ++it) {
-        if (pred(*it))
-            return *it;
         if (TreeItem *found = (*it)->reverseFindAnyChild(pred))
             return found;
+        if (pred(*it))
+            return *it;
     }
     return nullptr;
 }
@@ -878,8 +875,8 @@ void TreeItem::clear()
 {
     while (childCount() != 0) {
         TreeItem *item = m_children.takeLast();
-        item->m_model = 0;
-        item->m_parent = 0;
+        item->m_model = nullptr;
+        item->m_parent = nullptr;
         delete item;
     }
 }
@@ -899,10 +896,16 @@ void TreeItem::expand()
     m_model->requestExpansion(index());
 }
 
+void TreeItem::collapse()
+{
+    QTC_ASSERT(m_model, return);
+    m_model->requestCollapse(index());
+}
+
 void TreeItem::propagateModel(BaseTreeModel *m)
 {
     QTC_ASSERT(m, return);
-    QTC_ASSERT(m_model == 0 || m_model == m, return);
+    QTC_ASSERT(m_model == nullptr || m_model == m, return);
     if (m && !m_model) {
         m_model = m;
         for (TreeItem *item : *this)
@@ -939,9 +942,9 @@ BaseTreeModel::BaseTreeModel(TreeItem *root, QObject *parent)
 BaseTreeModel::~BaseTreeModel()
 {
     QTC_ASSERT(m_root, return);
-    QTC_ASSERT(m_root->m_parent == 0, return);
+    QTC_ASSERT(m_root->m_parent == nullptr, return);
     QTC_ASSERT(m_root->m_model == this, return);
-    m_root->m_model = 0;
+    m_root->m_model = nullptr;
     delete m_root;
 }
 
@@ -1024,6 +1027,8 @@ QVariant BaseTreeModel::headerData(int section, Qt::Orientation orientation,
 
 bool BaseTreeModel::hasChildren(const QModelIndex &idx) const
 {
+    if (idx.column() > 0)
+        return false;
     TreeItem *item = itemForIndex(idx);
     return !item || item->hasChildren();
 }
@@ -1031,7 +1036,7 @@ bool BaseTreeModel::hasChildren(const QModelIndex &idx) const
 Qt::ItemFlags BaseTreeModel::flags(const QModelIndex &idx) const
 {
     if (!idx.isValid())
-        return 0;
+        return nullptr;
     TreeItem *item = itemForIndex(idx);
     return item ? item->flags(idx.column())
                 : (Qt::ItemIsEnabled|Qt::ItemIsSelectable);
@@ -1062,19 +1067,19 @@ TreeItem *BaseTreeModel::rootItem() const
 void BaseTreeModel::setRootItem(TreeItem *item)
 {
     QTC_ASSERT(item, return);
-    QTC_ASSERT(item->m_model == 0, return);
-    QTC_ASSERT(item->m_parent == 0, return);
+    QTC_ASSERT(item->m_model == nullptr, return);
+    QTC_ASSERT(item->m_parent == nullptr, return);
     QTC_ASSERT(item != m_root, return);
     QTC_CHECK(m_root);
 
     beginResetModel();
     if (m_root) {
-        QTC_CHECK(m_root->m_parent == 0);
+        QTC_CHECK(m_root->m_parent == nullptr);
         QTC_CHECK(m_root->m_model == this);
         // needs to be done explicitly before setting the model to 0, otherwise it might lead to a
         // crash inside a view or proxy model, especially if there are selected items
         m_root->removeChildren();
-        m_root->m_model = 0;
+        m_root->m_model = nullptr;
         delete m_root;
     }
     m_root = item;
@@ -1110,8 +1115,8 @@ TreeItem *BaseTreeModel::itemForIndex(const QModelIndex &idx) const
 {
     CHECK_INDEX(idx);
     TreeItem *item = idx.isValid() ? static_cast<TreeItem*>(idx.internalPointer()) : m_root;
-    QTC_ASSERT(item, return 0);
-    QTC_ASSERT(item->m_model == static_cast<const BaseTreeModel *>(this), return 0);
+    QTC_ASSERT(item, return nullptr);
+    QTC_ASSERT(item->m_model == static_cast<const BaseTreeModel *>(this), return nullptr);
     return item;
 }
 
@@ -1124,7 +1129,7 @@ QModelIndex BaseTreeModel::indexForItem(const TreeItem *item) const
     TreeItem *p = item->parent();
     QTC_ASSERT(p, return QModelIndex());
 
-    TreeItem *mitem = const_cast<TreeItem *>(item);
+    auto mitem = const_cast<TreeItem *>(item);
     int row = p->indexOf(mitem);
     return createIndex(row, 0, mitem);
 }
@@ -1158,8 +1163,8 @@ TreeItem *BaseTreeModel::takeItem(TreeItem *item)
 
     QModelIndex idx = indexForItem(parent);
     beginRemoveRows(idx, pos, pos);
-    item->m_parent = 0;
-    item->m_model = 0;
+    item->m_parent = nullptr;
+    item->m_model = nullptr;
     parent->m_children.removeAt(pos);
     endRemoveRows();
     return item;
