@@ -33,7 +33,6 @@
 #include "androiddeployqtstep.h"
 #include "androidqtsupport.h"
 #include "androidqtversion.h"
-#include "androidbuildapkstep.h"
 #include "androidavdmanager.h"
 #include "androidsdkmanager.h"
 
@@ -49,27 +48,63 @@
 
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
+
 #include <utils/algorithm.h>
+#include <utils/qtcassert.h>
 #include <utils/synchronousprocess.h>
 
 #include <QDir>
 #include <QFileSystemWatcher>
 #include <QList>
+#include <QLoggingCategory>
 #include <QProcess>
 #include <QRegExp>
 #include <QMessageBox>
 #include <QApplication>
 #include <QDomDocument>
 #include <QVersionNumber>
+#include <QRegularExpression>
 
 namespace {
     const QLatin1String AndroidManifestName("AndroidManifest.xml");
     const QLatin1String AndroidDefaultPropertiesName("project.properties");
     const QLatin1String AndroidDeviceSn("AndroidDeviceSerialNumber");
     const QLatin1String ApiLevelKey("AndroidVersion.ApiLevel");
+    const QString packageNameRegEx("(package: name=)\\'(([a-z]{1}[a-z\\d_]*\\."
+                                   ")*[a-z][a-z\\d_]*)\\'");
+    const QString activityRegEx("(launchable-activity: name=)\\'"
+                                "(([a-z]{1}[a-z\\d_]*\\.)*[a-z][a-z\\d_]*)\\'");
+    const QString apkVersionRegEx("package: name=([\\=a-z\\d_\\.\\'\\s]*)"
+                                  "\\sversionName='([\\d\\.]*)'");
 
+    Q_LOGGING_CATEGORY(androidManagerLog, "qtc.android.androidManager", QtWarningMsg)
+
+    bool runCommand(const QString &executable, const QStringList &args,
+                    QString *output = nullptr, int timeoutS = 30)
+    {
+        Utils::SynchronousProcess cmdProc;
+        cmdProc.setTimeoutS(timeoutS);
+        qCDebug(androidManagerLog) << executable << args.join(' ');
+        Utils::SynchronousProcessResponse response = cmdProc.runBlocking(executable, args);
+        if (output)
+            *output = response.allOutput();
+        return response.result == Utils::SynchronousProcessResponse::Finished;
+    }
+
+    QString parseAaptOutput(const QString &output, const QString &regEx) {
+        const QRegularExpression regRx(regEx,
+                                       QRegularExpression::CaseInsensitiveOption |
+                                       QRegularExpression::MultilineOption);
+        QRegularExpressionMatch match = regRx.match(output);
+        if (match.hasMatch())
+            return match.captured(2);
+        return QString();
+    };
 } // anonymous namespace
 
+
+using namespace Utils;
+
 namespace Android {
 
 using namespace Internal;
@@ -84,7 +119,7 @@ public:
     QString name;
 };
 
-typedef QMap<QString, Library> LibrariesMap;
+using LibrariesMap = QMap<QString, Library>;
 
 static bool openXmlFile(QDomDocument &doc, const Utils::FileName &fileName);
 static bool openManifest(ProjectExplorer::Target *target, QDomDocument &doc);
@@ -120,6 +155,53 @@ QString AndroidManager::packageName(const Utils::FileName &manifestFile)
     return manifestElem.attribute(QLatin1String("package"));
 }
 
+bool AndroidManager::packageInstalled(const QString &deviceSerial,
+                                      const QString &packageName)
+{
+    if (deviceSerial.isEmpty() || packageName.isEmpty())
+        return false;
+    QStringList args = AndroidDeviceInfo::adbSelector(deviceSerial);
+    args << "shell" << "pm" << "list" << "packages";
+    QString output;
+    runAdbCommand(args, &output);
+    QStringList lines = output.split(QRegularExpression("[\\n\\r]"),
+                                     QString::SkipEmptyParts);
+    for (const QString &line : lines) {
+        // Don't want to confuse com.abc.xyz with com.abc.xyz.def so check with
+        // endsWith
+        if (line.endsWith(packageName))
+            return true;
+    }
+    return false;
+}
+
+void AndroidManager::apkInfo(const Utils::FileName &apkPath,
+                                    QString *packageName,
+                                    QVersionNumber *version,
+                                    QString *activityPath)
+{
+    QString output;
+    runAaptCommand({"dump", "badging", apkPath.toString()}, &output);
+
+    QString packageStr;
+    if (activityPath) {
+        packageStr = parseAaptOutput(output, packageNameRegEx);
+        QString path = parseAaptOutput(output, activityRegEx);
+        if (!packageStr.isEmpty() && !path.isEmpty())
+            *activityPath = packageStr + '/' + path;
+    }
+
+    if (packageName) {
+        *packageName = activityPath ? packageStr :
+                                      parseAaptOutput(output, packageNameRegEx);
+    }
+
+    if (version) {
+        QString versionStr = parseAaptOutput(output, apkVersionRegEx);
+        *version = QVersionNumber::fromString(versionStr);
+    }
+}
+
 QString AndroidManager::intentName(ProjectExplorer::Target *target)
 {
     return packageName(target) + QLatin1Char('/') + activityName(target);
@@ -136,7 +218,7 @@ QString AndroidManager::activityName(ProjectExplorer::Target *target)
 
 /*!
     Returns the minimum Android API level set for the APK. Minimum API level
-    of the kit is returned if the manifest file of the APK can not be found
+    of the kit is returned if the manifest file of the APK cannot be found
     or parsed.
 */
 int AndroidManager::minimumSDK(ProjectExplorer::Target *target)
@@ -169,7 +251,7 @@ int AndroidManager::minimumSDK(const ProjectExplorer::Kit *kit)
 
 QString AndroidManager::buildTargetSDK(ProjectExplorer::Target *target)
 {
-    AndroidBuildApkStep *androidBuildApkStep
+    auto androidBuildApkStep
             = AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
     if (androidBuildApkStep)
         return androidBuildApkStep->buildTargetSdk();
@@ -181,7 +263,7 @@ QString AndroidManager::buildTargetSDK(ProjectExplorer::Target *target)
 
 bool AndroidManager::signPackage(ProjectExplorer::Target *target)
 {
-    AndroidBuildApkStep *androidBuildApkStep
+    auto androidBuildApkStep
             = AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
     if (androidBuildApkStep)
         return androidBuildApkStep->signPackage();
@@ -190,23 +272,47 @@ bool AndroidManager::signPackage(ProjectExplorer::Target *target)
 
 QString AndroidManager::targetArch(ProjectExplorer::Target *target)
 {
-    AndroidQtVersion *qt = static_cast<AndroidQtVersion *>(QtSupport::QtKitInformation::qtVersion(target->kit()));
+    auto qt = static_cast<AndroidQtVersion *>(QtSupport::QtKitInformation::qtVersion(target->kit()));
     return qt->targetArch();
 }
 
-Utils::FileName AndroidManager::dirPath(ProjectExplorer::Target *target)
+Utils::FileName AndroidManager::dirPath(const ProjectExplorer::Target *target)
 {
     if (target->activeBuildConfiguration())
         return target->activeBuildConfiguration()->buildDirectory().appendPath(QLatin1String(Constants::ANDROID_BUILDDIRECTORY));
     return Utils::FileName();
 }
 
+Utils::FileName AndroidManager::apkPath(const ProjectExplorer::Target *target)
+{
+    QTC_ASSERT(target, return Utils::FileName());
+
+    auto buildApkStep
+        = Android::AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
+
+    if (!buildApkStep)
+        return Utils::FileName();
+
+    QString apkPath("build/outputs/apk/android-build-");
+    if (buildApkStep->signPackage())
+        apkPath += QLatin1String("release.apk");
+    else
+        apkPath += QLatin1String("debug.apk");
+
+    return dirPath(target).appendPath(apkPath);
+}
+
 Utils::FileName AndroidManager::manifestSourcePath(ProjectExplorer::Target *target)
 {
-    AndroidQtSupport *androidQtSupport = AndroidManager::androidQtSupport(target);
-    Utils::FileName source = androidQtSupport->manifestSourcePath(target);
-    if (!source.isEmpty())
-        return source;
+    if (AndroidQtSupport *androidQtSupport = AndroidManager::androidQtSupport(target)) {
+        const QString packageSource
+                = androidQtSupport->targetData(Android::Constants::AndroidPackageSourceDir, target).toString();
+        if (!packageSource.isEmpty()) {
+            const FileName manifest = FileName::fromUserInput(packageSource + "/AndroidManifest.xml");
+            if (manifest.exists())
+                return manifest;
+        }
+    }
     return manifestPath(target);
 }
 
@@ -222,7 +328,7 @@ Utils::FileName AndroidManager::defaultPropertiesPath(ProjectExplorer::Target *t
 
 bool AndroidManager::bundleQt(ProjectExplorer::Target *target)
 {
-    AndroidBuildApkStep *androidBuildApkStep
+    auto androidBuildApkStep
             = AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
     if (androidBuildApkStep)
         return !androidBuildApkStep->useMinistro();
@@ -237,12 +343,26 @@ QString AndroidManager::deviceSerialNumber(ProjectExplorer::Target *target)
 
 void AndroidManager::setDeviceSerialNumber(ProjectExplorer::Target *target, const QString &deviceSerialNumber)
 {
+    qCDebug(androidManagerLog) << "Device serial for the target changed"
+                               << target->displayName() << deviceSerialNumber;
     target->setNamedSettings(AndroidDeviceSn, deviceSerialNumber);
 }
 
+int AndroidManager::deviceApiLevel(ProjectExplorer::Target *target)
+{
+    return target->namedSettings(ApiLevelKey).toInt();
+}
+
+void AndroidManager::setDeviceApiLevel(ProjectExplorer::Target *target, int level)
+{
+    qCDebug(androidManagerLog) << "Device API level for the target changed"
+                               << target->displayName() << level;
+    target->setNamedSettings(ApiLevelKey, level);
+}
+
 QPair<int, int> AndroidManager::apiLevelRange()
 {
-    return qMakePair(9, 26);
+    return qMakePair(16, 28);
 }
 
 QString AndroidManager::androidNameForApiLevel(int x)
@@ -294,6 +414,10 @@ QString AndroidManager::androidNameForApiLevel(int x)
         return QLatin1String("Android 7.1");
     case 26:
         return QLatin1String("Android 8.0");
+    case 27:
+        return QLatin1String("Android 8.1");
+    case 28:
+        return QLatin1String("Android 9");
     default:
         return tr("Unknown Android version. API Level: %1").arg(QString::number(x));
     }
@@ -301,7 +425,7 @@ QString AndroidManager::androidNameForApiLevel(int x)
 
 static void raiseError(const QString &reason)
 {
-    QMessageBox::critical(0, AndroidManager::tr("Error creating Android templates."), reason);
+    QMessageBox::critical(nullptr, AndroidManager::tr("Error creating Android templates."), reason);
 }
 
 static bool openXmlFile(QDomDocument &doc, const Utils::FileName &fileName)
@@ -354,16 +478,9 @@ void AndroidManager::cleanLibsOnDevice(ProjectExplorer::Target *target)
             Core::MessageManager::write(tr("Starting Android virtual device failed."));
     }
 
-    QProcess *process = new QProcess();
     QStringList arguments = AndroidDeviceInfo::adbSelector(deviceSerialNumber);
     arguments << QLatin1String("shell") << QLatin1String("rm") << QLatin1String("-r") << QLatin1String("/data/local/tmp/qt");
-    QObject::connect(process, static_cast<void (QProcess::*)(int)>(&QProcess::finished),
-                     process, &QObject::deleteLater);
-    const QString adb = AndroidConfigurations::currentConfig().adbToolPath().toString();
-    Core::MessageManager::write(adb + QLatin1Char(' ') + arguments.join(QLatin1Char(' ')));
-    process->start(adb, arguments);
-    if (!process->waitForStarted(500))
-        delete process;
+    runAdbCommandDetached(arguments);
 }
 
 void AndroidManager::installQASIPackage(ProjectExplorer::Target *target, const QString &packagePath)
@@ -383,17 +500,9 @@ void AndroidManager::installQASIPackage(ProjectExplorer::Target *target, const Q
             Core::MessageManager::write(tr("Starting Android virtual device failed."));
     }
 
-    QProcess *process = new QProcess();
     QStringList arguments = AndroidDeviceInfo::adbSelector(deviceSerialNumber);
     arguments << QLatin1String("install") << QLatin1String("-r ") << packagePath;
-
-    connect(process, static_cast<void (QProcess::*)(int)>(&QProcess::finished),
-            process, &QObject::deleteLater);
-    const QString adb = AndroidConfigurations::currentConfig().adbToolPath().toString();
-    Core::MessageManager::write(adb + QLatin1Char(' ') + arguments.join(QLatin1Char(' ')));
-    process->start(adb, arguments);
-    if (!process->waitForStarted(500) && process->state() != QProcess::Running)
-        delete process;
+    runAdbCommandDetached(arguments);
 }
 
 bool AndroidManager::checkKeystorePassword(const QString &keystorePath, const QString &keystorePasswd)
@@ -443,27 +552,16 @@ bool AndroidManager::checkCertificateExists(const QString &keystorePath,
     return response.result == Utils::SynchronousProcessResponse::Finished && response.exitCode == 0;
 }
 
-bool AndroidManager::checkForQt51Files(Utils::FileName fileName)
-{
-    fileName.appendPath(QLatin1String("android")).appendPath(QLatin1String("version.xml"));
-    if (!fileName.exists())
-        return false;
-    QDomDocument dstVersionDoc;
-    if (!openXmlFile(dstVersionDoc, fileName))
-        return false;
-    return dstVersionDoc.documentElement().attribute(QLatin1String("value")).toDouble() < 5.2;
-}
-
 AndroidQtSupport *AndroidManager::androidQtSupport(ProjectExplorer::Target *target)
 {
     for (AndroidQtSupport *provider : g_androidQtSupportProviders) {
         if (provider->canHandle(target))
             return provider;
     }
-    return 0;
+    return nullptr;
 }
 
-typedef QMap<QByteArray, QByteArray> GradleProperties;
+using GradleProperties = QMap<QByteArray, QByteArray>;
 
 static GradleProperties readGradleProperties(const QString &path)
 {
@@ -531,13 +629,16 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target)
     if (!version)
         return false;
 
-    AndroidBuildApkStep *buildApkStep
-        = AndroidGlobal::buildStep<AndroidBuildApkStep>(target->activeBuildConfiguration());
-
-    if (!buildApkStep || !buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("gradlew")).exists())
+    AndroidQtSupport *qtSupport = androidQtSupport(target);
+    if (!qtSupport)
         return false;
 
-    Utils::FileName wrapperProps(buildApkStep->androidPackageSourceDir());
+    QFileInfo sourceDirInfo(qtSupport->targetData(Constants::AndroidPackageSourceDir, target).toString());
+    FileName packageSourceDir = FileName::fromString(sourceDirInfo.canonicalFilePath());
+    if (!packageSourceDir.appendPath("gradlew").exists())
+        return false;
+
+    Utils::FileName wrapperProps = packageSourceDir;
     wrapperProps.appendPath(QLatin1String("gradle/wrapper/gradle-wrapper.properties"));
     if (wrapperProps.exists()) {
         GradleProperties wrapperProperties = readGradleProperties(wrapperProps.toString());
@@ -551,10 +652,10 @@ bool AndroidManager::updateGradleProperties(ProjectExplorer::Target *target)
 
     GradleProperties localProperties;
     localProperties["sdk.dir"] = AndroidConfigurations::currentConfig().sdkLocation().toString().toLocal8Bit();
-    if (!mergeGradleProperties(buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("local.properties")).toString(), localProperties))
+    if (!mergeGradleProperties(packageSourceDir.appendPath("local.properties").toString(), localProperties))
         return false;
 
-    QString gradlePropertiesPath = buildApkStep->androidPackageSourceDir().appendPath(QLatin1String("gradle.properties")).toString();
+    QString gradlePropertiesPath = packageSourceDir.appendPath("gradle.properties").toString();
     GradleProperties gradleProperties = readGradleProperties(gradlePropertiesPath);
     gradleProperties["qt5AndroidDir"] = version->qmakeProperty("QT_INSTALL_PREFIX")
             .append(QLatin1String("/src/android/java")).toLocal8Bit();
@@ -584,4 +685,27 @@ int AndroidManager::findApiLevel(const Utils::FileName &platformPath)
     return apiLevel;
 }
 
+void AndroidManager::runAdbCommandDetached(const QStringList &args)
+{
+    auto process = new QProcess();
+    connect(process, static_cast<void (QProcess::*)(int)>(&QProcess::finished),
+            process, &QObject::deleteLater);
+    const QString adb = AndroidConfigurations::currentConfig().adbToolPath().toString();
+    qCDebug(androidManagerLog) << adb << args.join(' ');
+    process->start(adb, args);
+    if (!process->waitForStarted(500) && process->state() != QProcess::Running)
+        delete process;
+}
+
+bool AndroidManager::runAdbCommand(const QStringList &args, QString *output)
+{
+    return runCommand(AndroidConfigurations::currentConfig().adbToolPath().toString(),
+                      args, output);
+}
+
+bool AndroidManager::runAaptCommand(const QStringList &args, QString *output)
+{
+    return runCommand(AndroidConfigurations::currentConfig().aaptToolPath().toString(),
+               args, output);
+}
 } // namespace Android
