@@ -25,6 +25,7 @@
 
 #include "cppfindreferences.h"
 
+#include "cppfilesettingspage.h"
 #include "cpptoolsconstants.h"
 #include "cppmodelmanager.h"
 #include "cppworkingcopy.h"
@@ -33,6 +34,9 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
+#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
 #include <texteditor/basefilefind.h>
 
 #include <utils/algorithm.h>
@@ -42,6 +46,7 @@
 
 #include <cplusplus/Overview.h>
 #include <QtConcurrentMap>
+#include <QCheckBox>
 #include <QDir>
 
 #include <functional>
@@ -49,7 +54,7 @@
 using namespace Core;
 using namespace CppTools::Internal;
 using namespace CppTools;
-using namespace CPlusPlus;
+using namespace ProjectExplorer;
 
 static QByteArray getSource(const Utils::FileName &fileName,
                             const WorkingCopy &workingCopy)
@@ -70,7 +75,7 @@ static QByteArray getSource(const Utils::FileName &fileName,
     }
 }
 
-static QByteArray typeId(Symbol *symbol)
+static QByteArray typeId(CPlusPlus::Symbol *symbol)
 {
     if (symbol->asEnum()) {
         return QByteArray("e");
@@ -92,7 +97,7 @@ static QByteArray typeId(Symbol *symbol)
         return QByteArray("ud");
     } else if (symbol->asDeclaration()) {
         QByteArray temp("d,");
-        Overview pretty;
+        CPlusPlus::Overview pretty;
         temp.append(pretty.prettyType(symbol->type()).toUtf8());
         return temp;
     } else if (symbol->asArgument()) {
@@ -127,19 +132,19 @@ static QByteArray typeId(Symbol *symbol)
     return QByteArray("unknown");
 }
 
-static QByteArray idForSymbol(Symbol *symbol)
+static QByteArray idForSymbol(CPlusPlus::Symbol *symbol)
 {
     QByteArray uid(typeId(symbol));
-    if (const Identifier *id = symbol->identifier()) {
+    if (const CPlusPlus::Identifier *id = symbol->identifier()) {
         uid.append("|");
         uid.append(QByteArray(id->chars(), id->size()));
-    } else if (Scope *scope = symbol->enclosingScope()) {
+    } else if (CPlusPlus::Scope *scope = symbol->enclosingScope()) {
         // add the index of this symbol within its enclosing scope
         // (counting symbols without identifier of the same type)
         int count = 0;
-        Scope::iterator it = scope->memberBegin();
+        CPlusPlus::Scope::iterator it = scope->memberBegin();
         while (it != scope->memberEnd() && *it != symbol) {
-            Symbol *val = *it;
+            CPlusPlus::Symbol *val = *it;
             ++it;
             if (val->identifier() || typeId(val) != uid)
                 continue;
@@ -150,10 +155,10 @@ static QByteArray idForSymbol(Symbol *symbol)
     return uid;
 }
 
-static QList<QByteArray> fullIdForSymbol(Symbol *symbol)
+static QList<QByteArray> fullIdForSymbol(CPlusPlus::Symbol *symbol)
 {
     QList<QByteArray> uid;
-    Symbol *current = symbol;
+    CPlusPlus::Symbol *current = symbol;
     do {
         uid.prepend(idForSymbol(current));
         current = current->enclosingScope();
@@ -163,20 +168,24 @@ static QList<QByteArray> fullIdForSymbol(Symbol *symbol)
 
 namespace {
 
-class ProcessFile: public std::unary_function<QString, QList<Usage> >
+class ProcessFile
 {
     const WorkingCopy workingCopy;
-    const Snapshot snapshot;
-    Document::Ptr symbolDocument;
-    Symbol *symbol;
-    QFutureInterface<Usage> *future;
+    const CPlusPlus::Snapshot snapshot;
+    CPlusPlus::Document::Ptr symbolDocument;
+    CPlusPlus::Symbol *symbol;
+    QFutureInterface<CPlusPlus::Usage> *future;
 
 public:
+    // needed by QtConcurrent
+    using argument_type = const Utils::FileName &;
+    using result_type = QList<CPlusPlus::Usage>;
+
     ProcessFile(const WorkingCopy &workingCopy,
-                const Snapshot snapshot,
-                Document::Ptr symbolDocument,
-                Symbol *symbol,
-                QFutureInterface<Usage> *future)
+                const CPlusPlus::Snapshot snapshot,
+                CPlusPlus::Document::Ptr symbolDocument,
+                CPlusPlus::Symbol *symbol,
+                QFutureInterface<CPlusPlus::Usage> *future)
         : workingCopy(workingCopy),
           snapshot(snapshot),
           symbolDocument(symbolDocument),
@@ -184,21 +193,21 @@ public:
           future(future)
     { }
 
-    QList<Usage> operator()(const Utils::FileName &fileName)
+    QList<CPlusPlus::Usage> operator()(const Utils::FileName &fileName)
     {
-        QList<Usage> usages;
+        QList<CPlusPlus::Usage> usages;
         if (future->isPaused())
             future->waitForResume();
         if (future->isCanceled())
             return usages;
-        const Identifier *symbolId = symbol->identifier();
+        const CPlusPlus::Identifier *symbolId = symbol->identifier();
 
-        if (Document::Ptr previousDoc = snapshot.document(fileName)) {
-            Control *control = previousDoc->control();
+        if (CPlusPlus::Document::Ptr previousDoc = snapshot.document(fileName)) {
+            CPlusPlus::Control *control = previousDoc->control();
             if (!control->findIdentifier(symbolId->chars(), symbolId->size()))
                 return usages; // skip this document, it's not using symbolId.
         }
-        Document::Ptr doc;
+        CPlusPlus::Document::Ptr doc;
         const QByteArray unpreprocessedSource = getSource(fileName, workingCopy);
 
         if (symbolDocument && fileName == Utils::FileName::fromString(symbolDocument->fileName())) {
@@ -208,12 +217,12 @@ public:
             doc->tokenize();
         }
 
-        Control *control = doc->control();
+        CPlusPlus::Control *control = doc->control();
         if (control->findIdentifier(symbolId->chars(), symbolId->size()) != 0) {
             if (doc != symbolDocument)
                 doc->check();
 
-            FindUsages process(unpreprocessedSource, doc, snapshot);
+            CPlusPlus::FindUsages process(unpreprocessedSource, doc, snapshot);
             process(symbol);
 
             usages = process.usages();
@@ -225,16 +234,16 @@ public:
     }
 };
 
-class UpdateUI: public std::binary_function<QList<Usage> &, QList<Usage>, void>
+class UpdateUI
 {
-    QFutureInterface<Usage> *future;
+    QFutureInterface<CPlusPlus::Usage> *future;
 
 public:
-    UpdateUI(QFutureInterface<Usage> *future): future(future) {}
+    UpdateUI(QFutureInterface<CPlusPlus::Usage> *future): future(future) {}
 
-    void operator()(QList<Usage> &, const QList<Usage> &usages)
+    void operator()(QList<CPlusPlus::Usage> &, const QList<CPlusPlus::Usage> &usages)
     {
-        foreach (const Usage &u, usages)
+        foreach (const CPlusPlus::Usage &u, usages)
             future->reportResult(u);
 
         future->setProgressValue(future->progressValue() + 1);
@@ -253,26 +262,27 @@ CppFindReferences::~CppFindReferences()
 {
 }
 
-QList<int> CppFindReferences::references(Symbol *symbol, const LookupContext &context) const
+QList<int> CppFindReferences::references(CPlusPlus::Symbol *symbol,
+                                         const CPlusPlus::LookupContext &context) const
 {
     QList<int> references;
 
-    FindUsages findUsages(context);
+    CPlusPlus::FindUsages findUsages(context);
     findUsages(symbol);
     references = findUsages.references();
 
     return references;
 }
 
-static void find_helper(QFutureInterface<Usage> &future,
+static void find_helper(QFutureInterface<CPlusPlus::Usage> &future,
                         const WorkingCopy workingCopy,
-                        const LookupContext context,
-                        Symbol *symbol)
+                        const CPlusPlus::LookupContext context,
+                        CPlusPlus::Symbol *symbol)
 {
-    const Identifier *symbolId = symbol->identifier();
+    const CPlusPlus::Identifier *symbolId = symbol->identifier();
     QTC_ASSERT(symbolId != 0, return);
 
-    const Snapshot snapshot = context.snapshot();
+    const CPlusPlus::Snapshot snapshot = context.snapshot();
 
     const Utils::FileName sourceFile = Utils::FileName::fromUtf8(symbol->fileName(),
                                                                  symbol->fileNameLength());
@@ -283,12 +293,12 @@ static void find_helper(QFutureInterface<Usage> &future,
         || (symbol->enclosingScope()
             && !symbol->isStatic()
             && symbol->enclosingScope()->isNamespace())) {
-        const Snapshot snapshotFromContext = context.snapshot();
+        const CPlusPlus::Snapshot snapshotFromContext = context.snapshot();
         for (auto i = snapshotFromContext.begin(), ei = snapshotFromContext.end(); i != ei; ++i) {
             if (i.key() == sourceFile)
                 continue;
 
-            const Control *control = i.value()->control();
+            const CPlusPlus::Control *control = i.value()->control();
 
             if (control->findIdentifier(symbolId->chars(), symbolId->size()))
                 files.append(i.key());
@@ -305,22 +315,23 @@ static void find_helper(QFutureInterface<Usage> &future,
     // This thread waits for blockingMappedReduced to finish, so reduce the pool's used thread count
     // so the blockingMappedReduced can use one more thread, and increase it again afterwards.
     QThreadPool::globalInstance()->releaseThread();
-    QtConcurrent::blockingMappedReduced<QList<Usage> > (files, process, reduce);
+    QtConcurrent::blockingMappedReduced<QList<CPlusPlus::Usage> > (files, process, reduce);
     QThreadPool::globalInstance()->reserveThread();
     future.setProgressValue(files.size());
 }
 
-void CppFindReferences::findUsages(Symbol *symbol, const LookupContext &context)
+void CppFindReferences::findUsages(CPlusPlus::Symbol *symbol,
+                                   const CPlusPlus::LookupContext &context)
 {
     findUsages(symbol, context, QString(), false);
 }
 
-void CppFindReferences::findUsages(Symbol *symbol,
-                                   const LookupContext &context,
+void CppFindReferences::findUsages(CPlusPlus::Symbol *symbol,
+                                   const CPlusPlus::LookupContext &context,
                                    const QString &replacement,
                                    bool replace)
 {
-    Overview overview;
+    CPlusPlus::Overview overview;
     SearchResult *search = SearchResultWindow::instance()->startNewSearch(tr("C++ Usages:"),
                                                 QString(),
                                                 overview.prettyName(context.fullyQualifiedName(symbol)),
@@ -336,33 +347,42 @@ void CppFindReferences::findUsages(Symbol *symbol,
     CppFindReferencesParameters parameters;
     parameters.symbolId = fullIdForSymbol(symbol);
     parameters.symbolFileName = QByteArray(symbol->fileName());
+
+    if (symbol->isClass() || symbol->isForwardClassDeclaration()) {
+        CPlusPlus::Overview overview;
+        parameters.prettySymbolName = overview.prettyName(context.path(symbol).last());
+    }
+
     search->setUserData(qVariantFromValue(parameters));
     findAll_helper(search, symbol, context);
 }
 
-void CppFindReferences::renameUsages(Symbol *symbol, const LookupContext &context,
+void CppFindReferences::renameUsages(CPlusPlus::Symbol *symbol,
+                                     const CPlusPlus::LookupContext &context,
                                      const QString &replacement)
 {
-    if (const Identifier *id = symbol->identifier()) {
+    if (const CPlusPlus::Identifier *id = symbol->identifier()) {
         const QString textToReplace = replacement.isEmpty()
                 ? QString::fromUtf8(id->chars(), id->size()) : replacement;
         findUsages(symbol, context, textToReplace, true);
     }
 }
 
-void CppFindReferences::findAll_helper(SearchResult *search, Symbol *symbol,
-                                       const LookupContext &context)
+void CppFindReferences::findAll_helper(SearchResult *search, CPlusPlus::Symbol *symbol,
+                                       const CPlusPlus::LookupContext &context)
 {
     if (!(symbol && symbol->identifier())) {
         search->finishSearch(false);
         return;
     }
     connect(search, &SearchResult::activated,
-            this, &CppFindReferences::openEditor);
+            [](const SearchResultItem& item) {
+                Core::EditorManager::openEditorAtSearchResult(item);
+            });
 
     SearchResultWindow::instance()->popup(IOutputPane::ModeSwitch | IOutputPane::WithFocus);
     const WorkingCopy workingCopy = m_modelManager->workingCopy();
-    QFuture<Usage> result;
+    QFuture<CPlusPlus::Usage> result;
     result = Utils::runAsync(m_modelManager->sharedThreadPool(), find_helper,
                              workingCopy, context, symbol);
     createWatcher(result, search);
@@ -373,6 +393,11 @@ void CppFindReferences::findAll_helper(SearchResult *search, Symbol *symbol,
     connect(progress, &FutureProgress::clicked, search, &SearchResult::popup);
 }
 
+static bool isAllLowerCase(const QString &text)
+{
+    return text.toLower() == text;
+}
+
 void CppFindReferences::onReplaceButtonClicked(const QString &text,
                                                const QList<SearchResultItem> &items,
                                                bool preserveCase)
@@ -382,16 +407,71 @@ void CppFindReferences::onReplaceButtonClicked(const QString &text,
         m_modelManager->updateSourceFiles(fileNames.toSet());
         SearchResultWindow::instance()->hide();
     }
+
+    auto search = qobject_cast<SearchResult *>(sender());
+    QTC_ASSERT(search, return);
+
+    CppFindReferencesParameters parameters = search->userData().value<CppFindReferencesParameters>();
+    if (parameters.filesToRename.isEmpty())
+        return;
+
+    auto renameFilesCheckBox = qobject_cast<QCheckBox *>(search->additionalReplaceWidget());
+    if (!renameFilesCheckBox || !renameFilesCheckBox->isChecked())
+        return;
+
+    CppFileSettings settings;
+    settings.fromSettings(Core::ICore::settings());
+
+    const QStringList newPaths =
+            Utils::transform<QList>(parameters.filesToRename,
+                                    [&parameters, text, &settings](const Node *node) -> QString {
+        const QFileInfo fi = node->filePath().toFileInfo();
+        const QString oldSymbolName = parameters.prettySymbolName;
+        const QString oldBaseName = fi.baseName();
+        const QString newSymbolName = text;
+        QString newBaseName = newSymbolName;
+
+        // 1) new symbol lowercase: new base name lowercase
+        if (isAllLowerCase(newSymbolName)) {
+            newBaseName = newSymbolName;
+
+        // 2) old base name mixed case: new base name is verbatim symbol name
+        } else if (!isAllLowerCase(oldBaseName)) {
+            newBaseName = newSymbolName;
+
+        // 3) old base name lowercase, old symbol mixed case: new base name lowercase
+        } else if (!isAllLowerCase(oldSymbolName)) {
+            newBaseName = newSymbolName.toLower();
+
+        // 4) old base name lowercase, old symbol lowercase, new symbol mixed case:
+        //    use the preferences setting for new base name case
+        } else if (settings.lowerCaseFiles) {
+            newBaseName = newSymbolName.toLower();
+        }
+
+        if (newBaseName == oldBaseName)
+            return QString();
+
+        return fi.absolutePath() + "/" + newBaseName + '.' + fi.completeSuffix();
+    });
+
+    for (int i = 0; i < parameters.filesToRename.size(); ++i) {
+        if (!newPaths.at(i).isEmpty()) {
+            Node *node = parameters.filesToRename.at(i);
+            ProjectExplorerPlugin::renameFile(node, newPaths.at(i));
+        }
+    }
 }
 
 void CppFindReferences::searchAgain()
 {
     SearchResult *search = qobject_cast<SearchResult *>(sender());
     CppFindReferencesParameters parameters = search->userData().value<CppFindReferencesParameters>();
-    Snapshot snapshot = CppModelManager::instance()->snapshot();
+    parameters.filesToRename.clear();
+    CPlusPlus::Snapshot snapshot = CppModelManager::instance()->snapshot();
     search->restart();
-    LookupContext context;
-    Symbol *symbol = findSymbol(parameters, snapshot, &context);
+    CPlusPlus::LookupContext context;
+    CPlusPlus::Symbol *symbol = findSymbol(parameters, snapshot, &context);
     if (!symbol) {
         search->finishSearch(false);
         return;
@@ -400,13 +480,13 @@ void CppFindReferences::searchAgain()
 }
 
 namespace {
-class UidSymbolFinder : public SymbolVisitor
+class UidSymbolFinder : public CPlusPlus::SymbolVisitor
 {
 public:
     UidSymbolFinder(const QList<QByteArray> &uid) : m_uid(uid), m_index(0), m_result(0) { }
-    Symbol *result() const { return m_result; }
+    CPlusPlus::Symbol *result() const { return m_result; }
 
-    bool preVisit(Symbol *symbol)
+    bool preVisit(CPlusPlus::Symbol *symbol)
     {
         if (m_result)
             return false;
@@ -425,7 +505,7 @@ public:
         return true;
     }
 
-    void postVisit(Symbol *symbol)
+    void postVisit(CPlusPlus::Symbol *symbol)
     {
         if (symbol->asScope())
             --m_index;
@@ -434,23 +514,24 @@ public:
 private:
     QList<QByteArray> m_uid;
     int m_index;
-    Symbol *m_result;
+    CPlusPlus::Symbol *m_result;
 };
 }
 
-Symbol *CppFindReferences::findSymbol(const CppFindReferencesParameters &parameters,
-                                      const Snapshot &snapshot, LookupContext *context)
+CPlusPlus::Symbol *CppFindReferences::findSymbol(const CppFindReferencesParameters &parameters,
+                                                 const CPlusPlus::Snapshot &snapshot,
+                                                 CPlusPlus::LookupContext *context)
 {
     QTC_ASSERT(context, return 0);
     QString symbolFile = QLatin1String(parameters.symbolFileName);
     if (!snapshot.contains(symbolFile))
         return 0;
 
-    Document::Ptr newSymbolDocument = snapshot.document(symbolFile);
+    CPlusPlus::Document::Ptr newSymbolDocument = snapshot.document(symbolFile);
     // document is not parsed and has no bindings yet, do it
     QByteArray source = getSource(Utils::FileName::fromString(newSymbolDocument->fileName()),
                                   m_modelManager->workingCopy());
-    Document::Ptr doc =
+    CPlusPlus::Document::Ptr doc =
             snapshot.preprocessedDocument(source, newSymbolDocument->fileName());
     doc->check();
 
@@ -458,58 +539,90 @@ Symbol *CppFindReferences::findSymbol(const CppFindReferencesParameters &paramet
     UidSymbolFinder finder(parameters.symbolId);
     finder.accept(doc->globalNamespace());
     if (finder.result()) {
-        *context = LookupContext(doc, snapshot);
+        *context = CPlusPlus::LookupContext(doc, snapshot);
         return finder.result();
     }
     return 0;
 }
 
-static void displayResults(SearchResult *search, QFutureWatcher<Usage> *watcher,
+static void displayResults(SearchResult *search, QFutureWatcher<CPlusPlus::Usage> *watcher,
                            int first, int last)
 {
+    CppFindReferencesParameters parameters = search->userData().value<CppFindReferencesParameters>();
+
     for (int index = first; index != last; ++index) {
-        Usage result = watcher->future().resultAt(index);
-        search->addResult(result.path,
+        CPlusPlus::Usage result = watcher->future().resultAt(index);
+        search->addResult(result.path.toString(),
                           result.line,
                           result.lineText,
                           result.col,
                           result.len);
+
+        if (parameters.prettySymbolName.isEmpty())
+            continue;
+
+        if (Utils::contains(parameters.filesToRename, Utils::equal(&Node::filePath, result.path)))
+            continue;
+
+        Node *node = ProjectTree::nodeForFile(result.path);
+        if (!node) // Not part of any project
+            continue;
+
+        const QFileInfo fi = node->filePath().toFileInfo();
+        if (fi.baseName().compare(parameters.prettySymbolName, Qt::CaseInsensitive) == 0)
+            parameters.filesToRename.append(node);
     }
+
+    search->setUserData(qVariantFromValue(parameters));
 }
 
-void CppFindReferences::openEditor(const SearchResultItem &item)
+static void searchFinished(SearchResult *search, QFutureWatcher<CPlusPlus::Usage> *watcher)
 {
-    if (item.path.size() > 0) {
-        EditorManager::openEditorAt(QDir::fromNativeSeparators(item.path.first()),
-                                    item.mainRange.begin.line,
-                                    item.mainRange.begin.column);
-    } else {
-        EditorManager::openEditor(QDir::fromNativeSeparators(item.text));
-    }
-}
+    search->finishSearch(watcher->isCanceled());
 
+    CppFindReferencesParameters parameters = search->userData().value<CppFindReferencesParameters>();
+    if (!parameters.filesToRename.isEmpty()) {
+        const QStringList filesToRename
+                = Utils::transform<QList>(parameters.filesToRename, [](const Node *node) {
+            return node->filePath().toUserOutput();
+        });
+
+        auto renameCheckBox = qobject_cast<QCheckBox *>(search->additionalReplaceWidget());
+        if (renameCheckBox) {
+            renameCheckBox->setText(CppFindReferences::tr("Re&name %n files", nullptr, filesToRename.size()));
+            renameCheckBox->setToolTip(CppFindReferences::tr("Files:\n%1").arg(filesToRename.join('\n')));
+            renameCheckBox->setVisible(true);
+        }
+    }
+
+    watcher->deleteLater();
+}
 
 namespace {
 
-class FindMacroUsesInFile: public std::unary_function<QString, QList<Usage> >
+class FindMacroUsesInFile
 {
     const WorkingCopy workingCopy;
-    const Snapshot snapshot;
-    const Macro &macro;
-    QFutureInterface<Usage> *future;
+    const CPlusPlus::Snapshot snapshot;
+    const CPlusPlus::Macro &macro;
+    QFutureInterface<CPlusPlus::Usage> *future;
 
 public:
+    // needed by QtConcurrent
+    using argument_type = const Utils::FileName &;
+    using result_type = QList<CPlusPlus::Usage>;
+
     FindMacroUsesInFile(const WorkingCopy &workingCopy,
-                        const Snapshot snapshot,
-                        const Macro &macro,
-                        QFutureInterface<Usage> *future)
+                        const CPlusPlus::Snapshot snapshot,
+                        const CPlusPlus::Macro &macro,
+                        QFutureInterface<CPlusPlus::Usage> *future)
         : workingCopy(workingCopy), snapshot(snapshot), macro(macro), future(future)
     { }
 
-    QList<Usage> operator()(const Utils::FileName &fileName)
+    QList<CPlusPlus::Usage> operator()(const Utils::FileName &fileName)
     {
-        QList<Usage> usages;
-        Document::Ptr doc = snapshot.document(fileName);
+        QList<CPlusPlus::Usage> usages;
+        CPlusPlus::Document::Ptr doc = snapshot.document(fileName);
         QByteArray source;
 
 restart_search:
@@ -519,8 +632,8 @@ restart_search:
             return usages;
 
         usages.clear();
-        foreach (const Document::MacroUse &use, doc->macroUses()) {
-            const Macro &useMacro = use.macro();
+        foreach (const CPlusPlus::Document::MacroUse &use, doc->macroUses()) {
+            const CPlusPlus::Macro &useMacro = use.macro();
 
             if (useMacro.fileName() == macro.fileName()) { // Check if this is a match, but possibly against an outdated document.
                 if (source.isEmpty())
@@ -536,7 +649,7 @@ restart_search:
                 if (macro.name() == useMacro.name()) {
                     unsigned column;
                     const QString &lineSource = matchingLine(use.bytesBegin(), source, &column);
-                    usages.append(Usage(fileName.toString(), lineSource, use.beginLine(), column,
+                    usages.append(CPlusPlus::Usage(fileName, lineSource, use.beginLine(), column,
                                         useMacro.nameToQString().size()));
                 }
             }
@@ -562,7 +675,7 @@ restart_search:
             const char *currentSourceByte = utf8Source.constData() + lineBegin;
             unsigned char yychar = *currentSourceByte;
             while (currentSourceByte != startOfUse)
-                Lexer::yyinp_utf8(currentSourceByte, yychar, *columnOfUseStart);
+                CPlusPlus::Lexer::yyinp_utf8(currentSourceByte, yychar, *columnOfUseStart);
         }
 
         const QByteArray matchingLine = utf8Source.mid(lineBegin, lineEnd - lineBegin);
@@ -572,10 +685,10 @@ restart_search:
 
 } // end of anonymous namespace
 
-static void findMacroUses_helper(QFutureInterface<Usage> &future,
+static void findMacroUses_helper(QFutureInterface<CPlusPlus::Usage> &future,
                                  const WorkingCopy workingCopy,
-                                 const Snapshot snapshot,
-                                 const Macro macro)
+                                 const CPlusPlus::Snapshot snapshot,
+                                 const CPlusPlus::Macro macro)
 {
     const Utils::FileName sourceFile = Utils::FileName::fromString(macro.fileName());
     Utils::FileNameList files{sourceFile};
@@ -587,17 +700,18 @@ static void findMacroUses_helper(QFutureInterface<Usage> &future,
     // This thread waits for blockingMappedReduced to finish, so reduce the pool's used thread count
     // so the blockingMappedReduced can use one more thread, and increase it again afterwards.
     QThreadPool::globalInstance()->releaseThread();
-    QtConcurrent::blockingMappedReduced<QList<Usage> > (files, process, reduce);
+    QtConcurrent::blockingMappedReduced<QList<CPlusPlus::Usage> > (files, process, reduce);
     QThreadPool::globalInstance()->reserveThread();
     future.setProgressValue(files.size());
 }
 
-void CppFindReferences::findMacroUses(const Macro &macro)
+void CppFindReferences::findMacroUses(const CPlusPlus::Macro &macro)
 {
     findMacroUses(macro, QString(), false);
 }
 
-void CppFindReferences::findMacroUses(const Macro &macro, const QString &replacement, bool replace)
+void CppFindReferences::findMacroUses(const CPlusPlus::Macro &macro, const QString &replacement,
+                                      bool replace)
 {
     SearchResult *search = SearchResultWindow::instance()->startNewSearch(
                 tr("C++ Macro Usages:"),
@@ -615,9 +729,11 @@ void CppFindReferences::findMacroUses(const Macro &macro, const QString &replace
     SearchResultWindow::instance()->popup(IOutputPane::ModeSwitch | IOutputPane::WithFocus);
 
     connect(search, &SearchResult::activated,
-            this, &CppFindReferences::openEditor);
+            [](const Core::SearchResultItem& item) {
+                Core::EditorManager::openEditorAtSearchResult(item);
+            });
 
-    const Snapshot snapshot = m_modelManager->snapshot();
+    const CPlusPlus::Snapshot snapshot = m_modelManager->snapshot();
     const WorkingCopy workingCopy = m_modelManager->workingCopy();
 
     // add the macro definition itself
@@ -631,7 +747,7 @@ void CppFindReferences::findMacroUses(const Macro &macro, const QString &replace
                           macro.nameToQString().length());
     }
 
-    QFuture<Usage> result;
+    QFuture<CPlusPlus::Usage> result;
     result = Utils::runAsync(m_modelManager->sharedThreadPool(), findMacroUses_helper,
                              workingCopy, snapshot, macro);
     createWatcher(result, search);
@@ -641,17 +757,19 @@ void CppFindReferences::findMacroUses(const Macro &macro, const QString &replace
     connect(progress, &FutureProgress::clicked, search, &SearchResult::popup);
 }
 
-void CppFindReferences::renameMacroUses(const Macro &macro, const QString &replacement)
+void CppFindReferences::renameMacroUses(const CPlusPlus::Macro &macro, const QString &replacement)
 {
     const QString textToReplace = replacement.isEmpty() ? macro.nameToQString() : replacement;
     findMacroUses(macro, textToReplace, true);
 }
 
-void CppFindReferences::createWatcher(const QFuture<Usage> &future, SearchResult *search)
+void CppFindReferences::createWatcher(const QFuture<CPlusPlus::Usage> &future, SearchResult *search)
 {
-    QFutureWatcher<Usage> *watcher = new QFutureWatcher<Usage>();
+    QFutureWatcher<CPlusPlus::Usage> *watcher = new QFutureWatcher<CPlusPlus::Usage>();
     // auto-delete:
-    connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
+    connect(watcher, &QFutureWatcherBase::finished, watcher, [search, watcher]() {
+                searchFinished(search, watcher);
+            });
 
     connect(watcher, &QFutureWatcherBase::resultsReadyAt, search,
             [search, watcher](int first, int last) {
