@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -32,6 +27,7 @@
 #include "ldparser.h"
 #include "task.h"
 #include "projectexplorerconstants.h"
+#include "buildmanager.h"
 
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorsettings.h>
@@ -41,19 +37,17 @@ using namespace ProjectExplorer;
 
 // opt. drive letter + filename: (2 brackets)
 static const char FILE_PATTERN[] = "(<command[ -]line>|([A-Za-z]:)?[^:]+):";
-static const char COMMAND_PATTERN[] = "^(.*[\\\\/])?([a-z0-9]+-[a-z0-9]+-[a-z0-9]+-)?(gcc|g\\+\\+)(-[0-9\\.]+)?(\\.exe)?: ";
+static const char COMMAND_PATTERN[] = "^(.*?[\\\\/])?([a-z0-9]+-[a-z0-9]+-[a-z0-9]+-)?(gcc|g\\+\\+)(-[0-9\\.]+)?(\\.exe)?: ";
 
 GccParser::GccParser()
 {
     setObjectName(QLatin1String("GCCParser"));
     m_regExp.setPattern(QLatin1Char('^') + QLatin1String(FILE_PATTERN)
                         + QLatin1String("(\\d+):(\\d+:)?\\s+((fatal |#)?(warning|error|note):?\\s)?([^\\s].+)$"));
-    m_regExp.setMinimal(true);
     QTC_CHECK(m_regExp.isValid());
 
     m_regExpIncluded.setPattern(QString::fromLatin1("\\bfrom\\s") + QLatin1String(FILE_PATTERN)
                                 + QLatin1String("(\\d+)(:\\d+)?[,:]?$"));
-    m_regExpIncluded.setMinimal(true);
     QTC_CHECK(m_regExpIncluded.isValid());
 
     // optional path with trailing slash
@@ -62,7 +56,6 @@ GccParser::GccParser()
     // optional trailing version number
     // optional .exe postfix
     m_regExpGccNames.setPattern(QLatin1String(COMMAND_PATTERN));
-    m_regExpGccNames.setMinimal(true);
     QTC_CHECK(m_regExpGccNames.isValid());
 
     appendOutputParser(new LdParser);
@@ -88,8 +81,11 @@ void GccParser::stdError(const QString &line)
                      -1 /* linenumber */,
                      Constants::TASK_CATEGORY_COMPILE));
         return;
-    } else if (m_regExpGccNames.indexIn(lne) > -1) {
-        QString description = lne.mid(m_regExpGccNames.matchedLength());
+    }
+
+    QRegularExpressionMatch match = m_regExpGccNames.match(lne);
+    if (match.hasMatch()) {
+        QString description = lne.mid(match.capturedLength());
         Task::TaskType type = Task::Error;
         if (description.startsWith(QLatin1String("warning: "))) {
             type = Task::Warning;
@@ -101,30 +97,36 @@ void GccParser::stdError(const QString &line)
                   -1, /* line */ Constants::TASK_CATEGORY_COMPILE);
         newTask(task);
         return;
-    } else if (m_regExp.indexIn(lne) > -1) {
-        Utils::FileName filename = Utils::FileName::fromUserInput(m_regExp.cap(1));
-        int lineno = m_regExp.cap(3).toInt();
+    }
+
+    match = m_regExp.match(lne);
+    if (match.hasMatch()) {
+        Utils::FileName filename = Utils::FileName::fromUserInput(match.captured(1));
+        int lineno = match.captured(3).toInt();
         Task::TaskType type = Task::Unknown;
-        QString description = m_regExp.cap(8);
-        if (m_regExp.cap(7) == QLatin1String("warning"))
+        QString description = match.captured(8);
+        if (match.captured(7) == QLatin1String("warning"))
             type = Task::Warning;
-        else if (m_regExp.cap(7) == QLatin1String("error") ||
+        else if (match.captured(7) == QLatin1String("error") ||
                  description.startsWith(QLatin1String("undefined reference to")) ||
                  description.startsWith(QLatin1String("multiple definition of")))
             type = Task::Error;
         // Prepend "#warning" or "#error" if that triggered the match on (warning|error)
         // We want those to show how the warning was triggered
-        if (m_regExp.cap(5).startsWith(QLatin1Char('#')))
-            description = m_regExp.cap(5) + description;
+        if (match.captured(5).startsWith(QLatin1Char('#')))
+            description = match.captured(5) + description;
 
         Task task(type, description, filename, lineno, Constants::TASK_CATEGORY_COMPILE);
         newTask(task);
         return;
-    } else if (m_regExpIncluded.indexIn(lne) > -1) {
+    }
+
+    match = m_regExpIncluded.match(lne);
+    if (match.hasMatch()) {
         newTask(Task(Task::Unknown,
                      lne.trimmed() /* description */,
-                     Utils::FileName::fromUserInput(m_regExpIncluded.cap(1)) /* filename */,
-                     m_regExpIncluded.cap(3).toInt() /* linenumber */,
+                     Utils::FileName::fromUserInput(match.captured(1)) /* filename */,
+                     match.captured(3).toInt() /* linenumber */,
                      Constants::TASK_CATEGORY_COMPILE));
         return;
     } else if (lne.startsWith(QLatin1Char(' '))) {
@@ -142,10 +144,16 @@ void GccParser::stdOutput(const QString &line)
     IOutputParser::stdOutput(line);
 }
 
+Core::Id GccParser::id()
+{
+    return Core::Id("ProjectExplorer.OutputParser.Gcc");
+}
+
 void GccParser::newTask(const Task &task)
 {
     doFlush();
     m_currentTask = task;
+    m_lines = 1;
 }
 
 void GccParser::doFlush()
@@ -154,7 +162,8 @@ void GccParser::doFlush()
         return;
     Task t = m_currentTask;
     m_currentTask.clear();
-    emit addTask(t);
+    emit addTask(t, m_lines, 1);
+    m_lines = 0;
 }
 
 void GccParser::amendDescription(const QString &desc, bool monospaced)
@@ -172,6 +181,7 @@ void GccParser::amendDescription(const QString &desc, bool monospaced)
         fr.format.setFontStyleHint(QFont::Monospace);
         m_currentTask.formats.append(fr);
     }
+    ++m_lines;
     return;
 }
 
@@ -181,7 +191,6 @@ void GccParser::amendDescription(const QString &desc, bool monospaced)
 #   include <QTest>
 
 #   include "projectexplorer.h"
-#   include "metatypedeclarations.h"
 #   include "outputparser_test.h"
 
 void ProjectExplorerPlugin::testGccOutputParsers_data()
@@ -190,7 +199,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
     QTest::addColumn<OutputParserTester::Channel>("inputChannel");
     QTest::addColumn<QString>("childStdOutLines");
     QTest::addColumn<QString>("childStdErrLines");
-    QTest::addColumn<QList<ProjectExplorer::Task> >("tasks");
+    QTest::addColumn<QList<Task> >("tasks");
     QTest::addColumn<QString>("outputLines");
 
     const Core::Id categoryCompile = Constants::TASK_CATEGORY_COMPILE;
@@ -198,18 +207,18 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
     QTest::newRow("pass-through stdout")
             << QString::fromLatin1("Sometext") << OutputParserTester::STDOUT
             << QString::fromLatin1("Sometext\n") << QString()
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
     QTest::newRow("pass-through stderr")
             << QString::fromLatin1("Sometext") << OutputParserTester::STDERR
             << QString() << QString::fromLatin1("Sometext\n")
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
 
     QTest::newRow("ar output")
             << QString::fromLatin1("../../../../x86/i686-unknown-linux-gnu/bin/i686-unknown-linux-gnu-ar: creating lib/libSkyView.a") << OutputParserTester::STDERR
             << QString() << QString::fromLatin1("../../../../x86/i686-unknown-linux-gnu/bin/i686-unknown-linux-gnu-ar: creating lib/libSkyView.a\n")
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
 
     QTest::newRow("GCCE error")
@@ -218,7 +227,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "/temp/test/untitled8/main.cpp:9: error: (Each undeclared identifier is reported only once for each function it appears in.)")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function `int main(int, char**)':"),
                         Utils::FileName::fromUserInput(QLatin1String("/temp/test/untitled8/main.cpp")), -1,
@@ -237,7 +246,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("/src/corelib/global/qglobal.h:1635: warning: inline function `QDebug qDebug()' used but never defined")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Warning,
                         QLatin1String("inline function `QDebug qDebug()' used but never defined"),
                         Utils::FileName::fromUserInput(QLatin1String("/src/corelib/global/qglobal.h")), 1635,
@@ -247,7 +256,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("main.cpp:7:2: warning: Some warning")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>() << Task(Task::Warning,
+            << (QList<Task>() << Task(Task::Warning,
                                                        QLatin1String("Some warning"),
                                                        Utils::FileName::fromUserInput(QLatin1String("main.cpp")), 7,
                                                        categoryCompile))
@@ -256,7 +265,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("C:\\temp\\test\\untitled8\\main.cpp:7: #error Symbian error")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("#error Symbian error"),
                         Utils::FileName::fromUserInput(QLatin1String("C:\\temp\\test\\untitled8\\main.cpp")), 7,
@@ -267,7 +276,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("C:\\temp\\test\\untitled8\\main.cpp:8: warning: #warning Symbian warning")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Warning,
                         QLatin1String("#warning Symbian warning"),
                         Utils::FileName::fromUserInput(QLatin1String("C:\\temp\\test\\untitled8\\main.cpp")), 8,
@@ -277,7 +286,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("/temp/test/untitled8/main.cpp:8:2: warning: #warning Symbian warning")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Warning,
                         QLatin1String("#warning Symbian warning"),
                         Utils::FileName::fromUserInput(QLatin1String("/temp/test/untitled8/main.cpp")), 8,
@@ -289,7 +298,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "collect2: ld returned 1 exit status")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function `main':"),
                         Utils::FileName::fromUserInput(QLatin1String("main.o")), -1,
@@ -310,7 +319,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "collect2: ld returned 1 exit status")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function `main':"),
                         Utils::FileName::fromUserInput(QLatin1String("main.o")), -1,
@@ -329,7 +338,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("c:\\Qt\\4.6\\lib/QtGuid4.dll: file not recognized: File format not recognized")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("file not recognized: File format not recognized"),
                         Utils::FileName::fromUserInput(QLatin1String("c:\\Qt\\4.6\\lib/QtGuid4.dll")), -1,
@@ -339,7 +348,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("g++: /usr/local/lib: No such file or directory")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("/usr/local/lib: No such file or directory"),
                         Utils::FileName(), -1,
@@ -352,7 +361,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "../../../../master/src/plugins/debugger/gdb/gdbengine.cpp:2115: warning: unused variable 'handler'")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In member function 'void Debugger::Internal::GdbEngine::handleBreakInsert2(const Debugger::Internal::GdbResponse&)':"),
                         Utils::FileName::fromUserInput(QLatin1String("../../../../master/src/plugins/debugger/gdb/gdbengine.cpp")), -1,
@@ -372,7 +381,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "/home/code/src/creator/src/plugins/projectexplorer/gnumakeparser.cpp:264: error: expected ';' before ':' token")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In member function 'void ProjectExplorer::ProjectExplorerPlugin::testGnuMakeParserTaskMangling_data()':"),
                         Utils::FileName::fromUserInput(QLatin1String("/home/code/src/creator/src/plugins/projectexplorer/gnumakeparser.cpp")), -1,
@@ -392,13 +401,13 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << OutputParserTester::STDERR
             << QString() << QString::fromLatin1("distcc[73168] (dcc_get_hostlist) Warning: no hostlist is set; can't distribute work\n"
                                                 "distcc[73168] (dcc_build_somewhere) Warning: failed to distribute, running locally instead\n")
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
     QTest::newRow("ld warning (QTCREATORBUG-905)")
             << QString::fromLatin1("ld: warning: Core::IEditor* QVariant::value<Core::IEditor*>() const has different visibility (hidden) in .obj/debug-shared/openeditorsview.o and (default) in .obj/debug-shared/editormanager.o")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Warning,
                          QLatin1String("Core::IEditor* QVariant::value<Core::IEditor*>() const has different visibility (hidden) in .obj/debug-shared/openeditorsview.o and (default) in .obj/debug-shared/editormanager.o"),
                          Utils::FileName(), -1,
@@ -408,7 +417,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("ld: fatal: Symbol referencing errors. No output written to testproject")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Error,
                          QLatin1String("Symbol referencing errors. No output written to testproject"),
                          Utils::FileName(), -1,
@@ -418,13 +427,13 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("TeamBuilder Client:: error: could not find Scheduler, running Job locally...")
             << OutputParserTester::STDERR
             << QString() << QString::fromLatin1("TeamBuilder Client:: error: could not find Scheduler, running Job locally...\n")
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
     QTest::newRow("note")
             << QString::fromLatin1("/home/dev/creator/share/qtcreator/debugger/dumper.cpp:1079: note: initialized from here")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("initialized from here"),
                          Utils::FileName::fromUserInput(QLatin1String("/home/dev/creator/share/qtcreator/debugger/dumper.cpp")), 1079,
@@ -435,7 +444,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "/Qt/4.6.2-Symbian/s60sdk/epoc32/include/stdapis/stlport/stl/_tree.c:194: warning: suggest explicit braces to avoid ambiguous 'else'")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("In static member function 'static std::_Rb_tree_node_base* std::_Rb_global<_Dummy>::_Rebalance_for_erase(std::_Rb_tree_node_base*, std::_Rb_tree_node_base*&, std::_Rb_tree_node_base*&, std::_Rb_tree_node_base*&)':"),
                          Utils::FileName::fromUserInput(QLatin1String("/Qt/4.6.2-Symbian/s60sdk/epoc32/include/stdapis/stlport/stl/_tree.c")), -1,
@@ -449,19 +458,13 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("rm: cannot remove `release/moc_mainwindow.cpp': No such file or directory")
             << OutputParserTester::STDERR
             << QString() << QString::fromLatin1("rm: cannot remove `release/moc_mainwindow.cpp': No such file or directory\n")
-            << QList<ProjectExplorer::Task>()
-            << QString();
-    QTest::newRow("ranlib false positive")
-            << QString::fromLatin1("ranlib: file: libSupport.a(HashTable.o) has no symbols")
-            << OutputParserTester::STDERR
-            << QString() << QString::fromLatin1("ranlib: file: libSupport.a(HashTable.o) has no symbols\n")
-            << QList<ProjectExplorer::Task>()
+            << QList<Task>()
             << QString();
     QTest::newRow("ld: missing library")
             << QString::fromLatin1("/usr/bin/ld: cannot find -ldoesnotexist")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Error,
                          QLatin1String("cannot find -ldoesnotexist"),
                          Utils::FileName(), -1,
@@ -473,7 +476,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "../../scriptbug/main.cpp:8: warning: unused variable c")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("In function void foo(i) [with i = double]:"),
                          Utils::FileName::fromUserInput(QLatin1String("../../scriptbug/main.cpp")), -1,
@@ -491,7 +494,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("main.cpp:10: instantiated from here  ")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("instantiated from here"),
                          Utils::FileName::fromUserInput(QLatin1String("main.cpp")), 10,
@@ -501,7 +504,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("/dev/creator/src/plugins/find/basetextfind.h: In constructor 'Find::BaseTextFind::BaseTextFind(QTextEdit*)':")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("In constructor 'Find::BaseTextFind::BaseTextFind(QTextEdit*)':"),
                          Utils::FileName::fromUserInput(QLatin1String("/dev/creator/src/plugins/find/basetextfind.h")), -1,
@@ -516,7 +519,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "../../scriptbug/main.cpp:5: warning: unused parameter v")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("At global scope:"),
                          Utils::FileName::fromUserInput(QLatin1String("../../scriptbug/main.cpp")), -1,
@@ -543,7 +546,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("/home/code/test.cpp:54:38: fatal error: test.moc: No such file or directory")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Error,
                          QLatin1String("test.moc: No such file or directory"),
                          Utils::FileName::fromUserInput(QLatin1String("/home/code/test.cpp")), 54,
@@ -557,7 +560,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "collect2: ld returned 1 exit status")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function `QPlotAxis':"),
                         Utils::FileName::fromUserInput(QLatin1String("debug/qplotaxis.o")), -1,
@@ -584,7 +587,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "../stl/main.cpp:31: warning: unused parameter index")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In member function typename _Vector_base<_Tp, _Alloc>::_Tp_alloc_type::const_reference Vector<_Tp, _Alloc>::at(int) [with _Tp = Point, _Alloc = Allocator<Point>]:"),
                         Utils::FileName::fromUserInput(QLatin1String("../stl/main.cpp")), -1,
@@ -614,7 +617,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "C:/Symbian_SDK/epoc32/include/e32cmn.inl:7094: warning: returning reference to temporary")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In file included from C:/Symbian_SDK/epoc32/include/e32cmn.h:6792,"),
                         Utils::FileName::fromUserInput(QLatin1String("C:/Symbian_SDK/epoc32/include/e32cmn.h")), 6792,
@@ -637,7 +640,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("../../../src/XmlUg/targetdelete.c: At top level:")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("At top level:"),
                          Utils::FileName::fromUserInput(QLatin1String("../../../src/XmlUg/targetdelete.c")), -1,
@@ -650,7 +653,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "/Symbian/SDK/epoc32/include/variant/Symbian_OS.hrh:1134:26: warning: no newline at end of file")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Unknown,
                          QLatin1String("In file included from /Symbian/SDK/EPOC32/INCLUDE/GCCE/GCCE.h:15,"),
                          Utils::FileName::fromUserInput(QLatin1String("/Symbian/SDK/EPOC32/INCLUDE/GCCE/GCCE.h")), 15,
@@ -669,7 +672,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("release/main.o:main.cpp:(.text+0x42): undefined reference to `MainWindow::doSomething()'")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("undefined reference to `MainWindow::doSomething()'"),
                         Utils::FileName::fromUserInput(QLatin1String("main.cpp")), -1,
@@ -681,7 +684,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "../../../src/shared/proparser/profileevaluator.cpp:2817:9: warning: case value '0' not in enumerated type 'ProFileEvaluator::Private::TestFunc'")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In member function 'ProFileEvaluator::Private::VisitReturn ProFileEvaluator::Private::evaluateConditionalFunction(const ProString&, const ProStringList&)':"),
                         Utils::FileName::fromUserInput(QLatin1String("../../../src/shared/proparser/profileevaluator.cpp")), -1,
@@ -697,7 +700,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "./mw.h:4:0: warning: \"STUPID_DEFINE\" redefined")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In file included from <command-line>:0:0:"),
                         Utils::FileName::fromUserInput(QLatin1String("<command-line>")), 0,
@@ -713,7 +716,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "file.h:21:5: warning: comparison between signed and unsigned integer expressions [-Wsign-compare]")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function 'void UnitTest::CheckEqual(UnitTest::TestResults&, const Expected&, const Actual&, const UnitTest::TestDetails&) [with Expected = unsigned int, Actual = int]':"),
                         Utils::FileName::fromUserInput(QLatin1String("file.h")), -1,
@@ -731,7 +734,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("cns5k_ins_parser_tests.cpp:(.text._ZN20CNS5kINSParserEngine21DropBytesUntilStartedEP14CircularBufferIhE[CNS5kINSParserEngine::DropBytesUntilStarted(CircularBuffer<unsigned char>*)]+0x6d): undefined reference to `CNS5kINSPacket::SOH_BYTE'")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("undefined reference to `CNS5kINSPacket::SOH_BYTE'"),
                         Utils::FileName::fromUserInput(QLatin1String("cns5k_ins_parser_tests.cpp")), -1,
@@ -742,7 +745,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("mainwindow.ui: Warning: The name 'pushButton' (QPushButton) is already in use, defaulting to 'pushButton1'.")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Warning,
                          QLatin1String("The name 'pushButton' (QPushButton) is already in use, defaulting to 'pushButton1'."),
                          Utils::FileName::fromUserInput(QLatin1String("mainwindow.ui")), -1,
@@ -753,7 +756,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
             << QString::fromLatin1("libimf.so: warning: warning: feupdateenv is not implemented and will always fail")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                  << Task(Task::Warning,
                          QLatin1String("warning: feupdateenv is not implemented and will always fail"),
                          Utils::FileName::fromUserInput(QLatin1String("libimf.so")), -1,
@@ -767,7 +770,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "                         ^")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In file included from /home/code/src/creator/src/libs/extensionsystem/pluginerrorview.cpp:31:0:"),
                         Utils::FileName::fromUserInput(QLatin1String("/home/code/src/creator/src/libs/extensionsystem/pluginerrorview.cpp")), 31,
@@ -788,7 +791,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "main.cpp:7:22: error: within this context")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << ( QList<ProjectExplorer::Task>()
+            << ( QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In file included from /usr/include/qt4/QtCore/QString:1:0,"),
                         Utils::FileName::fromUserInput(QLatin1String("/usr/include/qt4/QtCore/QString")), 1,
@@ -818,7 +821,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "collect2: error: ld returned 1 exit status")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Unknown,
                         QLatin1String("In function `foo()':"),
                         Utils::FileName::fromUserInput(QLatin1String("foo.o")), -1,
@@ -844,7 +847,7 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                                    "collect2: error: ld returned 1 exit status")
             << OutputParserTester::STDERR
             << QString() << QString()
-            << (QList<ProjectExplorer::Task>()
+            << (QList<Task>()
                 << Task(Task::Error,
                         QLatin1String("multiple definition of `foo'"),
                         Utils::FileName::fromUserInput(QLatin1String("foo.o")), -1,
@@ -859,6 +862,52 @@ void ProjectExplorerPlugin::testGccOutputParsers_data()
                         categoryCompile)
                 )
             << QString();
+
+    QTest::newRow("ld: undefined member function reference")
+            << "obj/gtest-clang-printing.o:gtest-clang-printing.cpp:llvm::VerifyDisableABIBreakingChecks: error: undefined reference to 'llvm::DisableABIBreakingChecks'"
+            << OutputParserTester::STDERR
+            << QString() << QString()
+            << (QList<Task>()
+                << Task(Task::Error,
+                        QLatin1String("error: undefined reference to 'llvm::DisableABIBreakingChecks'"),
+                        Utils::FileName::fromString("gtest-clang-printing.cpp"), -1,
+                        categoryCompile)
+                )
+            << QString();
+
+    QTest::newRow("Mac: ranlib warning")
+            << QString::fromLatin1("ranlib: file: lib/libtest.a(Test0.cpp.o) has no symbols")
+            << OutputParserTester::STDERR
+            << QString() << QString()
+            << (QList<Task>()
+                << Task(Task::Warning,
+                        QLatin1String("file: lib/libtest.a(Test0.cpp.o) has no symbols"),
+                        Utils::FileName(), -1,
+                        categoryCompile)
+                )
+            << QString();
+    QTest::newRow("Mac: ranlib warning2")
+            << QString::fromLatin1("/path/to/XCode/and/ranlib: file: lib/libtest.a(Test0.cpp.o) has no symbols")
+            << OutputParserTester::STDERR
+            << QString() << QString()
+            << (QList<Task>()
+                << Task(Task::Warning,
+                        QLatin1String("file: lib/libtest.a(Test0.cpp.o) has no symbols"),
+                        Utils::FileName(), -1,
+                        categoryCompile)
+                )
+            << QString();
+    QTest::newRow("moc note")
+            << QString::fromLatin1("/home/qtwebkithelpviewer.h:0: Note: No relevant classes found. No output generated.")
+            << OutputParserTester::STDERR
+            << QString() << QString()
+            << (QList<ProjectExplorer::Task>()
+                << Task(Task::Unknown,
+                        QLatin1String("Note: No relevant classes found. No output generated."),
+                        Utils::FileName::fromUserInput(QLatin1String("/home/qtwebkithelpviewer.h")), 0,
+                        categoryCompile)
+                )
+            << QString();
 }
 
 void ProjectExplorerPlugin::testGccOutputParsers()
