@@ -25,9 +25,11 @@
 
 #include "modelnodeoperations.h"
 #include "modelnodecontextmenu_helper.h"
+#include "addimagesdialog.h"
 #include "layoutingridlayout.h"
 #include "findimplementation.h"
 
+
 #include "addsignalhandlerdialog.h"
 
 #include <bindingproperty.h>
@@ -377,7 +379,7 @@ void resetZ(const SelectionContext &selectionState)
     }
 }
 
-static inline void backupPropertyAndRemove(ModelNode node, const PropertyName &propertyName)
+static inline void backupPropertyAndRemove(const ModelNode &node, const PropertyName &propertyName)
 {
     if (node.hasVariantProperty(propertyName)) {
         node.setAuxiliaryData(auxDataString + propertyName, node.variantProperty(propertyName).value());
@@ -442,7 +444,7 @@ void anchorsReset(const SelectionContext &selectionState)
     }
 }
 
-typedef std::function<bool(const ModelNode &node1, const ModelNode &node2)> LessThan;
+using LessThan = std::function<bool (const ModelNode &, const ModelNode&)>;
 
 bool compareByX(const ModelNode &node1, const ModelNode &node2)
 {
@@ -478,7 +480,7 @@ bool compareByGrid(const ModelNode &node1, const ModelNode &node2)
 
 
 static void layoutHelperFunction(const SelectionContext &selectionContext,
-                                 TypeName layoutType,
+                                 const TypeName &layoutType,
                                  LessThan lessThan)
 {
     if (!selectionContext.view()
@@ -569,22 +571,6 @@ void layoutGridLayout(const SelectionContext &selectionContext)
     }
 }
 
-/*
-bool optionsPageLessThan(const IOptionsPage *p1, const IOptionsPage *p2)
-{
-    if (p1->category() != p2->category())
-        return p1->category().alphabeticallyBefore(p2->category());
-    return p1->id().alphabeticallyBefore(p2->id());
-}
-
-static inline QList<IOptionsPage*> sortedOptionsPages()
-{
-    QList<IOptionsPage*> rc = ExtensionSystem::PluginManager::getObjects<IOptionsPage>();
-    qStableSort(rc.begin(), rc.end(), optionsPageLessThan);
-    return rc;
-}
-
-*/
 static PropertyNameList sortedPropertyNameList(const PropertyNameList &nameList)
 {
     PropertyNameList sortedPropertyNameList = nameList;
@@ -592,7 +578,7 @@ static PropertyNameList sortedPropertyNameList(const PropertyNameList &nameList)
     return sortedPropertyNameList;
 }
 
-static QString toUpper(const QString signal)
+static QString toUpper(const QString &signal)
 {
     QString ret = signal;
     ret[0] = signal.at(0).toUpper();
@@ -602,9 +588,9 @@ static QString toUpper(const QString signal)
 static void addSignal(const QString &typeName, const QString &itemId, const QString &signalName, bool isRootModelNode)
 {
     QScopedPointer<Model> model(Model::create("Item", 2, 0));
-    RewriterView rewriterView(RewriterView::Amend, 0);
+    RewriterView rewriterView(RewriterView::Amend, nullptr);
 
-    TextEditor::TextEditorWidget *textEdit = qobject_cast<TextEditor::TextEditorWidget*>
+    auto textEdit = qobject_cast<TextEditor::TextEditorWidget*>
             (Core::EditorManager::currentEditor()->widget());
 
     BaseTextEditModifier modifier(textEdit);
@@ -713,7 +699,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
         Core::EditorManager::openEditorAt(usages.constFirst().path, usages.constFirst().line, usages.constFirst().col);
 
         if (!signalNames.isEmpty()) {
-            AddSignalHandlerDialog *dialog = new AddSignalHandlerDialog(Core::ICore::dialogParent());
+            auto dialog = new AddSignalHandlerDialog(Core::ICore::dialogParent());
             dialog->setSignals(signalNames);
 
             AddSignalHandlerDialog::connect(dialog, &AddSignalHandlerDialog::signalSelected, [=] {
@@ -1024,25 +1010,35 @@ void addTabBarToStackedContainer(const SelectionContext &selectionContext)
     }
 }
 
-bool addImageToProject(const QString &fileName, const QString &directory)
+bool addImageToProject(const QStringList &fileNames, const QString &defaultDirectory)
 {
-    const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
-    const bool success = QFile::copy(fileName, targetFile);
+    QString directory = AddImagesDialog::getDirectory(fileNames, defaultDirectory);
 
-    auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
+    if (directory.isEmpty())
+        return true;
 
-    QTC_ASSERT(document, return false);
+    bool allSuccessful = true;
+    for (const QString &fileName : fileNames) {
+        const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
+        const bool success = QFile::copy(fileName, targetFile);
 
-    if (success) {
-        ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
-        if (node) {
-            ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
-            if (containingFolder)
-                containingFolder->addFiles(QStringList(targetFile));
+        auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
+
+        QTC_ASSERT(document, return false);
+
+        if (success) {
+            ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
+            if (node) {
+                ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
+                if (containingFolder)
+                    containingFolder->addFiles(QStringList(targetFile));
+            }
+        } else {
+            allSuccessful = false;
         }
     }
 
-    return success;
+    return allSuccessful;
 }
 
 } // namespace Mode
