@@ -176,9 +176,7 @@ NavigatorTreeModel::NavigatorTreeModel(QObject *parent) : QAbstractItemModel(par
 {
 }
 
-NavigatorTreeModel::~NavigatorTreeModel()
-{
-}
+NavigatorTreeModel::~NavigatorTreeModel() = default;
 
 QVariant NavigatorTreeModel::data(const QModelIndex &index, int role) const
 {
@@ -262,7 +260,7 @@ QModelIndex NavigatorTreeModel::index(int row, int column,
                                       const QModelIndex &parent) const
 {
     if (!m_view->model())
-        return QModelIndex();
+        return {};
 
     if (!hasIndex(row, column, parent))
         return QModelIndex();
@@ -290,7 +288,7 @@ QVariant NavigatorTreeModel::headerData(int, Qt::Orientation, int) const
 QModelIndex NavigatorTreeModel::parent(const QModelIndex &index) const
 {
     if (!index.isValid())
-        return QModelIndex();
+        return {};
 
     const ModelNode modelNode = modelNodeForIndex(index);
 
@@ -373,7 +371,7 @@ QStringList NavigatorTreeModel::mimeTypes() const
 
 QMimeData *NavigatorTreeModel::mimeData(const QModelIndexList &modelIndexList) const
 {
-    QMimeData *mimeData = new QMimeData();
+    auto mimeData = new QMimeData();
 
     QByteArray encodedModelNodeData;
     QDataStream encodedModelNodeDataStream(&encodedModelNodeData, QIODevice::WriteOnly);
