@@ -30,6 +30,7 @@
 #include "propertyeditortransaction.h"
 
 #include <qmldesignerconstants.h>
+#include <qmltimeline.h>
 #include <nodemetainfo.h>
 
 #include <invalididexception.h>
@@ -69,17 +70,20 @@ static bool propertyIsAttachedLayoutProperty(const PropertyName &propertyName)
 PropertyEditorView::PropertyEditorView(QWidget *parent) :
         AbstractView(parent),
         m_parent(parent),
-        m_updateShortcut(0),
+        m_updateShortcut(nullptr),
         m_timerId(0),
         m_stackedWidget(new PropertyEditorWidget(parent)),
-        m_qmlBackEndForCurrentType(0),
+        m_qmlBackEndForCurrentType(nullptr),
         m_locked(false),
         m_setupCompleted(false),
         m_singleShotTimer(new QTimer(this))
 {
     m_qmlDir = PropertyEditorQmlBackend::propertyEditorResourcesPath();
 
-    m_updateShortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F3), m_stackedWidget);
+    if (Utils::HostOsInfo().isMacHost())
+        m_updateShortcut = new QShortcut(QKeySequence(Qt::ALT + Qt::Key_F3), m_stackedWidget);
+    else
+        m_updateShortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F3), m_stackedWidget);
     connect(m_updateShortcut, &QShortcut::activated, this, &PropertyEditorView::reloadQml);
 
     m_stackedWidget->setStyleSheet(Theme::replaceCssColors(
@@ -135,7 +139,7 @@ void PropertyEditorView::changeValue(const QString &name)
     if (propertyName.isNull())
         return;
 
-    if (m_locked)
+    if (locked())
         return;
 
     if (propertyName == "className")
@@ -151,13 +155,13 @@ void PropertyEditorView::changeValue(const QString &name)
         if (newId == m_selectedNode.id())
             return;
 
-        if (m_selectedNode.isValidId(newId)  && !hasId(newId)) {
+        if (QmlDesigner::ModelNode::isValidId(newId)  && !hasId(newId)) {
             m_selectedNode.setIdWithRefactoring(newId);
         } else {
             m_locked = true;
             value->setValue(m_selectedNode.id());
             m_locked = false;
-            if (!m_selectedNode.isValidId(newId))
+            if (!QmlDesigner::ModelNode::isValidId(newId))
                 Core::AsynchronousMessageBox::warning(tr("Invalid Id"),  tr("%1 is an invalid id.").arg(newId));
             else
                 Core::AsynchronousMessageBox::warning(tr("Invalid Id"),  tr("%1 already exists.").arg(newId));
@@ -169,7 +173,7 @@ void PropertyEditorView::changeValue(const QString &name)
     underscoreName.replace('.', '_');
     PropertyEditorValue *value = m_qmlBackEndForCurrentType->propertyValueForName(QString::fromLatin1(underscoreName));
 
-    if (value ==0)
+    if (value ==nullptr)
         return;
 
     QmlObjectNode qmlObjectNode(m_selectedNode);
@@ -194,7 +198,8 @@ void PropertyEditorView::changeValue(const QString &name)
         if (qmlObjectNode.modelNode().metaInfo().propertyTypeName(propertyName) == "QUrl"
                 || qmlObjectNode.modelNode().metaInfo().propertyTypeName(propertyName) == "url") { //turn absolute local file paths into relative paths
                 QString filePath = castedValue.toUrl().toString();
-            if (QFileInfo(filePath).exists() && QFileInfo(filePath).isAbsolute()) {
+            QFileInfo fi(filePath);
+            if (fi.exists() && fi.isAbsolute()) {
                 QDir fileDir(QFileInfo(model()->fileUrl().toLocalFile()).absolutePath());
                 castedValue = QUrl(fileDir.relativeFilePath(filePath));
             }
@@ -231,7 +236,7 @@ void PropertyEditorView::changeExpression(const QString &propertyName)
     if (name.isNull())
         return;
 
-    if (m_locked)
+    if (locked())
         return;
 
     if (!m_selectedNode.isValid())
@@ -306,7 +311,7 @@ void PropertyEditorView::exportPopertyAsAlias(const QString &name)
     if (name.isNull())
         return;
 
-    if (m_locked)
+    if (locked())
         return;
 
     if (!m_selectedNode.isValid())
@@ -340,7 +345,7 @@ void PropertyEditorView::removeAliasExport(const QString &name)
     if (name.isNull())
         return;
 
-    if (m_locked)
+    if (locked())
         return;
 
     if (!m_selectedNode.isValid())
@@ -362,11 +367,24 @@ void PropertyEditorView::removeAliasExport(const QString &name)
     }
 }
 
+bool PropertyEditorView::locked() const
+{
+    return m_locked;
+}
+
+void PropertyEditorView::nodeCreated(const ModelNode &modelNode)
+{
+    if (!m_qmlBackEndForCurrentType->contextObject()->hasActiveTimeline()
+            && QmlTimeline::isValidQmlTimeline(modelNode)) {
+        m_qmlBackEndForCurrentType->contextObject()->setHasActiveTimeline(QmlTimeline::hasActiveTimeline(this));
+    }
+}
+
 void PropertyEditorView::updateSize()
 {
     if (!m_qmlBackEndForCurrentType)
         return;
-    QWidget* frame = m_qmlBackEndForCurrentType->widget()->findChild<QWidget*>("propertyEditorFrame");
+    auto frame = m_qmlBackEndForCurrentType->widget()->findChild<QWidget*>("propertyEditorFrame");
     if (frame)
         frame->resize(m_stackedWidget->size());
 }
@@ -397,7 +415,7 @@ void PropertyEditorView::timerEvent(QTimerEvent *timerEvent)
 
 void PropertyEditorView::resetView()
 {
-    if (model() == 0)
+    if (model() == nullptr)
         return;
 
     m_locked = true;
@@ -434,11 +452,7 @@ void PropertyEditorView::setupQmlBackend()
     TypeName diffClassName;
     if (m_selectedNode.isValid()) {
         diffClassName = m_selectedNode.metaInfo().typeName();
-        QList<NodeMetaInfo> hierarchy;
-        hierarchy << m_selectedNode.metaInfo();
-        hierarchy << m_selectedNode.metaInfo().superClasses();
-
-        foreach (const NodeMetaInfo &metaInfo, hierarchy) {
+        foreach (const NodeMetaInfo &metaInfo, m_selectedNode.metaInfo().classHierarchy()) {
             if (PropertyEditorQmlBackend::checkIfUrlExists(qmlSpecificsFile))
                 break;
             qmlSpecificsFile = PropertyEditorQmlBackend::getQmlFileUrl(metaInfo.typeName() + "Specifics", metaInfo);
@@ -508,8 +522,8 @@ void PropertyEditorView::selectedNodesChanged(const QList<ModelNode> &selectedNo
 
     if (selectedNodeList.isEmpty() || selectedNodeList.count() > 1)
         select(ModelNode());
-    else if (m_selectedNode != selectedNodeList.first())
-        select(selectedNodeList.first());
+    else if (m_selectedNode != selectedNodeList.constFirst())
+        select(selectedNodeList.constFirst());
 }
 
 void PropertyEditorView::nodeAboutToBeRemoved(const ModelNode &removedNode)
@@ -527,7 +541,6 @@ void PropertyEditorView::modelAttached(Model *model)
 
     m_locked = true;
 
-    resetView();
     if (!m_setupCompleted) {
         m_singleShotTimer->setSingleShot(true);
         m_singleShotTimer->setInterval(100);
@@ -536,6 +549,8 @@ void PropertyEditorView::modelAttached(Model *model)
     }
 
     m_locked = false;
+
+    resetView();
 }
 
 void PropertyEditorView::modelAboutToBeDetached(Model *model)
@@ -676,7 +691,7 @@ bool PropertyEditorView::hasWidget() const
 
 WidgetInfo PropertyEditorView::widgetInfo()
 {
-    return createWidgetInfo(m_stackedWidget, 0, QStringLiteral("Properties"), WidgetInfo::RightPane, 0);
+    return createWidgetInfo(m_stackedWidget, nullptr, QStringLiteral("Properties"), WidgetInfo::RightPane, 0);
 }
 
 void PropertyEditorView::currentStateChanged(const ModelNode &node)
@@ -694,7 +709,7 @@ void PropertyEditorView::instancePropertyChanged(const QList<QPair<ModelNode, Pr
         return;
     m_locked = true;
 
-    typedef QPair<ModelNode, PropertyName> ModelNodePropertyPair;
+    using ModelNodePropertyPair = QPair<ModelNode, PropertyName>;
     foreach (const ModelNodePropertyPair &propertyPair, propertyList) {
         const ModelNode modelNode = propertyPair.first;
         const QmlObjectNode qmlObjectNode(modelNode);
@@ -750,7 +765,7 @@ void PropertyEditorView::reloadQml()
         m_stackedWidget->removeWidget(widget);
         delete widget;
     }
-    m_qmlBackEndForCurrentType = 0;
+    m_qmlBackEndForCurrentType = nullptr;
 
     delayedResetView();
 }
