@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -32,6 +27,7 @@
 #include <metainfo.h>
 #include "qmlchangeset.h"
 #include "nodelistproperty.h"
+#include "nodehints.h"
 #include "variantproperty.h"
 #include "bindingproperty.h"
 #include "qmlanchors.h"
@@ -52,10 +48,13 @@ namespace QmlDesigner {
 
 bool QmlItemNode::isItemOrWindow(const ModelNode &modelNode)
 {
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Item", -1, -1))
+    if (modelNode.metaInfo().isSubclassOf("QtQuick.Item"))
         return true;
 
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Window.Window", -1, -1) && modelNode.isRootNode())
+    if (modelNode.metaInfo().isSubclassOf("QtQuick.Window.Window") && modelNode.isRootNode())
+        return true;
+
+    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Popup"))
         return true;
 
     return false;
@@ -70,7 +69,7 @@ static QmlItemNode createQmlItemNodeFromSource(AbstractView *view, const QString
     textEdit.setPlainText(source);
     NotIndentingTextEditModifier modifier(&textEdit);
 
-    QScopedPointer<RewriterView> rewriterView(new RewriterView(RewriterView::Amend, 0));
+    QScopedPointer<RewriterView> rewriterView(new RewriterView(RewriterView::Amend, nullptr));
     rewriterView->setCheckSemanticErrors(false);
     rewriterView->setTextModifier(&modifier);
     inputModel->setRewriterView(rewriterView.data());
@@ -99,7 +98,7 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibrary
 
     NodeAbstractProperty parentProperty = parentQmlItemNode.defaultNodeAbstractProperty();
 
-    return  QmlItemNode::createQmlItemNode(view, itemLibraryEntry, position, parentProperty);
+    return QmlItemNode::createQmlItemNode(view, itemLibraryEntry, position, parentProperty);
 }
 
 QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, NodeAbstractProperty parentproperty)
@@ -114,20 +113,23 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibrary
         int minorVersion = metaInfo.minorVersion();
         int majorVersion = metaInfo.majorVersion();
 
-        typedef QPair<PropertyName, QString> PropertyBindingEntry;
+        using PropertyBindingEntry = QPair<PropertyName, QString>;
         QList<PropertyBindingEntry> propertyBindingList;
+        QList<PropertyBindingEntry> propertyEnumList;
         if (itemLibraryEntry.qmlSource().isEmpty()) {
             QList<QPair<PropertyName, QVariant> > propertyPairList;
             if (!position.isNull()) {
-                propertyPairList.append(qMakePair(PropertyName("x"), QVariant(qRound(position.x()))));
-                propertyPairList.append(qMakePair(PropertyName("y"), QVariant(qRound(position.y()))));
+                propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
+                propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
             }
 
-            foreach (const PropertyContainer &property, itemLibraryEntry.properties()) {
+            for (const auto &property : itemLibraryEntry.properties()) {
                 if (property.type() == QStringLiteral("binding")) {
                     propertyBindingList.append(PropertyBindingEntry(property.name(), property.value().toString()));
+                } else if (property.type() == QStringLiteral("enum"))  {
+                    propertyEnumList.append(PropertyBindingEntry(property.name(), property.value().toString()));
                 } else {
-                    propertyPairList.append(qMakePair(property.name(), property.value()));
+                    propertyPairList.append({property.name(), property.value()});
                 }
             }
 
@@ -144,17 +146,15 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibrary
 
         newQmlItemNode.setId(view->generateNewId(itemLibraryEntry.name()));
 
-        if (!view->currentState().isBaseState()) {
-            newQmlItemNode.modelNode().variantProperty("opacity").setValue(0);
-            newQmlItemNode.setVariantProperty("opacity", 1);
-        }
-
-        foreach (const PropertyBindingEntry &propertyBindingEntry, propertyBindingList)
+        for (const auto &propertyBindingEntry : propertyBindingList)
             newQmlItemNode.modelNode().bindingProperty(propertyBindingEntry.first).setExpression(propertyBindingEntry.second);
 
+        for (const auto &propertyBindingEntry : propertyEnumList)
+            newQmlItemNode.modelNode().variantProperty(propertyBindingEntry.first).setEnumeration(propertyBindingEntry.second.toUtf8());
+
         Q_ASSERT(newQmlItemNode.isValid());
     }
-    catch (RewritingException &e) {
+    catch (const RewritingException &e) {
         e.showException();
     }
 
@@ -183,8 +183,8 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
         if (view->model()->hasNodeMetaInfo("QtQuick.Image")) {
             NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Image");
             QList<QPair<PropertyName, QVariant> > propertyPairList;
-            propertyPairList.append(qMakePair(PropertyName("x"), QVariant(qRound(position.x()))));
-            propertyPairList.append(qMakePair(PropertyName("y"), QVariant(qRound(position.y()))));
+            propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
+            propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
 
             QString relativeImageName = imageName;
 
@@ -192,18 +192,15 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
             if (QFileInfo::exists(view->model()->fileUrl().toLocalFile())) {
                 QDir fileDir(QFileInfo(view->model()->fileUrl().toLocalFile()).absolutePath());
                 relativeImageName = fileDir.relativeFilePath(imageName);
-                propertyPairList.append(qMakePair(PropertyName("source"), QVariant(relativeImageName)));
+                propertyPairList.append({PropertyName("source"), QVariant(relativeImageName)});
             }
 
             newQmlItemNode = QmlItemNode(view->createModelNode("QtQuick.Image", metaInfo.majorVersion(), metaInfo.minorVersion(), propertyPairList));
             parentproperty.reparentHere(newQmlItemNode);
 
-            newQmlItemNode.setId(view->generateNewId("image"));
+            newQmlItemNode.setId(view->generateNewId(QLatin1String("image")));
 
-            if (!view->currentState().isBaseState()) {
-                newQmlItemNode.modelNode().variantProperty("opacity").setValue(0);
-                newQmlItemNode.setVariantProperty("opacity", 1);
-            }
+            newQmlItemNode.modelNode().variantProperty("fillMode").setEnumeration("Image.PreserveAspectFit");
 
             Q_ASSERT(newQmlItemNode.isValid());
         }
@@ -369,32 +366,54 @@ bool QmlItemNode::instanceHasRotationTransform() const
 
 bool itemIsMovable(const ModelNode &modelNode)
 {
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Tab", -1, -1))
+    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Tab"))
         return false;
 
-    if (modelNode.hasParentProperty()) {
-        ModelNode parentModelNode = modelNode.parentProperty().parentModelNode();
-        if (QmlItemNode::isValidQmlItemNode(parentModelNode)
-                && parentModelNode.metaInfo().isLayoutable())
-            return false;
-    }
+    if (!modelNode.hasParentProperty())
+        return false;
 
-    return true;
+    if (!modelNode.parentProperty().isNodeListProperty())
+        return false;
+
+    return NodeHints::fromModelNode(modelNode).isMovable();
 }
 
+bool itemIsResizable(const ModelNode &modelNode)
+{
+    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Tab"))
+        return false;
+
+    return NodeHints::fromModelNode(modelNode).isResizable();
+}
 
 bool QmlItemNode::modelIsMovable() const
 {
     return !modelNode().hasBindingProperty("x")
             && !modelNode().hasBindingProperty("y")
-            && itemIsMovable(modelNode());
+            && itemIsMovable(modelNode())
+            && !modelIsInLayout();
 }
 
 bool QmlItemNode::modelIsResizable() const
 {
     return !modelNode().hasBindingProperty("width")
             && !modelNode().hasBindingProperty("height")
-            && itemIsMovable(modelNode());
+            && itemIsResizable(modelNode())
+            && !modelIsInLayout();
+}
+
+bool QmlItemNode::modelIsInLayout() const
+{
+    if (modelNode().hasParentProperty()) {
+        ModelNode parentModelNode = modelNode().parentProperty().parentModelNode();
+        if (QmlItemNode::isValidQmlItemNode(parentModelNode)
+                && parentModelNode.metaInfo().isLayoutable())
+            return true;
+
+        return NodeHints::fromModelNode(parentModelNode).doesLayoutChildren();
+    }
+
+    return false;
 }
 
 QRectF  QmlItemNode::instanceBoundingRect() const
@@ -444,7 +463,7 @@ QPointF QmlItemNode::instanceScenePosition() const
      else if (modelNode().hasParentProperty() && QmlItemNode::isValidQmlItemNode(modelNode().parentProperty().parentModelNode()))
         return QmlItemNode(modelNode().parentProperty().parentModelNode()).instanceSceneTransform().map(nodeInstance().position());
 
-    return QPointF();
+    return {};
 }
 
 QPointF QmlItemNode::instancePosition() const
@@ -493,7 +512,7 @@ QList<QmlModelState> QmlModelStateGroup::allStates() const
     return returnList;
 }
 
-TypeName QmlItemNode::simplifiedTypeName() const
+QString QmlItemNode::simplifiedTypeName() const
 {
     return modelNode().simplifiedTypeName();
 }
@@ -508,11 +527,8 @@ QmlModelState QmlModelStateGroup::addState(const QString &name)
     if (!modelNode().isValid())
         throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
-
-    PropertyListType propertyList;
-    propertyList.append(qMakePair(PropertyName("name"), QVariant(name)));
-
-    ModelNode newState = QmlModelState::createQmlState(modelNode().view(), propertyList);
+    ModelNode newState = QmlModelState::createQmlState(
+        modelNode().view(), {{PropertyName("name"), QVariant(name)}});
     modelNode().nodeListProperty("states").reparentHere(newState);
 
     return newState;
@@ -597,12 +613,41 @@ void QmlItemNode::setPostionInBaseState(const QPointF &position)
     modelNode().variantProperty("y").setValue(qRound(position.y()));
 }
 
+bool QmlItemNode::isInLayout() const
+{
+    if (isValid() && hasNodeParent()) {
+
+        ModelNode parent = modelNode().parentProperty().parentModelNode();
+
+        if (parent.isValid() && parent.metaInfo().isValid())
+            return parent.metaInfo().isSubclassOf("QtQuick.Layouts.Layout");
+    }
+
+    return false;
+}
+
+bool QmlItemNode::canBereparentedTo(const ModelNode &potentialParent) const
+{
+    if (!NodeHints::fromModelNode(potentialParent).canBeContainerFor(modelNode()))
+        return false;
+    return NodeHints::fromModelNode(modelNode()).canBeReparentedTo(potentialParent);
+}
+
+bool QmlItemNode::isInStackedContainer() const
+{
+    if (hasInstanceParent())
+        return NodeHints::fromModelNode(instanceParent()).isStackedContainer();
+    return false;
+}
+
 void QmlItemNode::setSize(const QSizeF &size)
 {
-    if (!hasBindingProperty("width") && !anchors().instanceHasAnchor(AnchorLineRight))
+    if (!hasBindingProperty("width") && !(anchors().instanceHasAnchor(AnchorLineRight)
+                                          && anchors().instanceHasAnchor(AnchorLineLeft)))
         setVariantProperty("width", qRound(size.width()));
 
-    if (!hasBindingProperty("height") && !anchors().instanceHasAnchor(AnchorLineBottom))
+    if (!hasBindingProperty("height") && !(anchors().instanceHasAnchor(AnchorLineBottom)
+                                           && anchors().instanceHasAnchor(AnchorLineTop)))
         setVariantProperty("height", qRound(size.height()));
 }
 
