@@ -31,9 +31,11 @@
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/completionsettings.h>
 #include <texteditor/texteditorconstants.h>
+#include <texteditor/codeassist/assistproposaliteminterface.h>
 
 #include <utils/faketooltip.h>
 #include <utils/hostosinfo.h>
+#include <utils/utilsicons.h>
 
 #include <QRect>
 #include <QLatin1String>
@@ -49,52 +51,12 @@
 #include <QKeyEvent>
 #include <QDesktopWidget>
 #include <QLabel>
+#include <QStyledItemDelegate>
 
 using namespace Utils;
 
 namespace TextEditor {
 
-static QString cleanText(const QString &original)
-{
-    QString clean = original;
-    int ignore = 0;
-    for (int i = clean.length() - 1; i >= 0; --i, ++ignore) {
-        const QChar &c = clean.at(i);
-        if (c.isLetterOrNumber() || c == QLatin1Char('_')
-                || c.isHighSurrogate() || c.isLowSurrogate()) {
-            break;
-        }
-    }
-    if (ignore)
-        clean.chop(ignore);
-    return clean;
-}
-
-static bool isPerfectMatch(const QString &prefix, const GenericProposalModel *model)
-{
-    if (prefix.isEmpty())
-        return false;
-
-    for (int i = 0; i < model->size(); ++i) {
-        const QString &current = cleanText(model->text(i));
-        if (!current.isEmpty()) {
-            CaseSensitivity cs = TextEditorSettings::completionSettings().m_caseSensitivity;
-            if (cs == TextEditor::CaseSensitive) {
-                if (prefix == current)
-                    return true;
-            } else if (cs == TextEditor::CaseInsensitive) {
-                if (prefix.compare(current, Qt::CaseInsensitive) == 0)
-                    return true;
-            } else if (cs == TextEditor::FirstLetterCaseSensitive) {
-                if (prefix.at(0) == current.at(0)
-                        && prefix.midRef(1).compare(current.midRef(1), Qt::CaseInsensitive) == 0)
-                    return true;
-            }
-        }
-    }
-    return false;
-}
-
 // ------------
 // ModelAdapter
 // ------------
@@ -103,16 +65,16 @@ class ModelAdapter : public QAbstractListModel
     Q_OBJECT
 
 public:
-    ModelAdapter(GenericProposalModel *completionModel, QWidget *parent);
+    ModelAdapter(GenericProposalModelPtr completionModel, QWidget *parent);
 
-    virtual int rowCount(const QModelIndex &) const;
-    virtual QVariant data(const QModelIndex &index, int role) const;
+    int rowCount(const QModelIndex &) const override;
+    QVariant data(const QModelIndex &index, int role) const override;
 
 private:
-    GenericProposalModel *m_completionModel;
+    GenericProposalModelPtr m_completionModel;
 };
 
-ModelAdapter::ModelAdapter(GenericProposalModel *completionModel, QWidget *parent)
+ModelAdapter::ModelAdapter(GenericProposalModelPtr completionModel, QWidget *parent)
     : QAbstractListModel(parent)
     , m_completionModel(completionModel)
 {}
@@ -133,6 +95,8 @@ QVariant ModelAdapter::data(const QModelIndex &index, int role) const
         return m_completionModel->icon(index.row());
     else if (role == Qt::WhatsThisRole)
         return m_completionModel->detail(index.row());
+    else if (role == Qt::UserRole)
+        return m_completionModel->proposalItem(index.row())->requiresFixIts();
 
     return QVariant();
 }
@@ -143,10 +107,10 @@ QVariant ModelAdapter::data(const QModelIndex &index, int role) const
 class GenericProposalInfoFrame : public FakeToolTip
 {
 public:
-    GenericProposalInfoFrame(QWidget *parent = 0)
+    GenericProposalInfoFrame(QWidget *parent = nullptr)
         : FakeToolTip(parent), m_label(new QLabel(this))
     {
-        QVBoxLayout *layout = new QVBoxLayout(this);
+        auto layout = new QVBoxLayout(this);
         layout->setMargin(0);
         layout->setSpacing(0);
         layout->addWidget(m_label);
@@ -187,6 +151,7 @@ private:
 // -----------------------
 class GenericProposalListView : public QListView
 {
+    friend class ProposalItemDelegate;
 public:
     GenericProposalListView(QWidget *parent);
 
@@ -201,10 +166,53 @@ public:
     void selectLastRow() { selectRow(model()->rowCount() - 1); }
 };
 
+class ProposalItemDelegate : public QStyledItemDelegate
+{
+    Q_OBJECT
+public:
+    explicit ProposalItemDelegate(GenericProposalListView *parent = nullptr)
+        : QStyledItemDelegate(parent)
+        , m_parent(parent)
+    {
+    }
+
+    void paint(QPainter *painter,
+               const QStyleOptionViewItem &option,
+               const QModelIndex &index) const override
+    {
+        static const QIcon fixItIcon = ::Utils::Icons::CODEMODEL_FIXIT.icon();
+
+        QStyledItemDelegate::paint(painter, option, index);
+
+        if (m_parent->model()->data(index, Qt::UserRole).toBool()) {
+            const QRect itemRect = m_parent->rectForIndex(index);
+            const QScrollBar *verticalScrollBar = m_parent->verticalScrollBar();
+
+            const int x = m_parent->width() - itemRect.height() - (verticalScrollBar->isVisible()
+                                                                   ? verticalScrollBar->width()
+                                                                   : 0);
+            const int iconSize = itemRect.height() - 5;
+            fixItIcon.paint(painter, QRect(x, itemRect.y() - m_parent->verticalOffset(),
+                                           iconSize, iconSize));
+        }
+    }
+
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
+    {
+        QSize size(QStyledItemDelegate::sizeHint(option, index));
+        if (m_parent->model()->data(index, Qt::UserRole).toBool())
+            size.setWidth(size.width() + m_parent->rectForIndex(index).height() - 5);
+        return size;
+    }
+private:
+    GenericProposalListView *m_parent;
+};
+
 GenericProposalListView::GenericProposalListView(QWidget *parent)
     : QListView(parent)
 {
     setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
+    setItemDelegate(new ProposalItemDelegate(this));
 }
 
 QSize GenericProposalListView::calculateSize() const
@@ -247,33 +255,26 @@ class GenericProposalWidgetPrivate : public QObject
 public:
     GenericProposalWidgetPrivate(QWidget *completionWidget);
 
-    const QWidget *m_underlyingWidget;
+    const QWidget *m_underlyingWidget = nullptr;
     GenericProposalListView *m_completionListView;
-    GenericProposalModel *m_model;
+    GenericProposalModelPtr m_model;
     QRect m_displayRect;
-    bool m_isSynchronized;
-    bool m_explicitlySelected;
-    AssistReason m_reason;
-    AssistKind m_kind;
-    bool m_justInvoked;
+    bool m_isSynchronized = true;
+    bool m_explicitlySelected = false;
+    AssistReason m_reason = IdleEditor;
+    AssistKind m_kind = Completion;
+    bool m_justInvoked = false;
     QPointer<GenericProposalInfoFrame> m_infoFrame;
     QTimer m_infoTimer;
-    CodeAssistant *m_assistant;
-    bool m_autoWidth;
+    CodeAssistant *m_assistant = nullptr;
+    bool m_autoWidth = true;
 
     void handleActivation(const QModelIndex &modelIndex);
     void maybeShowInfoTip();
 };
 
 GenericProposalWidgetPrivate::GenericProposalWidgetPrivate(QWidget *completionWidget)
-    : m_underlyingWidget(0)
-    , m_completionListView(new GenericProposalListView(completionWidget))
-    , m_model(0)
-    , m_isSynchronized(true)
-    , m_explicitlySelected(false)
-    , m_justInvoked(false)
-    , m_assistant(0)
-    , m_autoWidth(true)
+    : m_completionListView(new GenericProposalListView(completionWidget))
 {
     m_completionListView->setIconSize(QSize(16, 16));
     connect(m_completionListView, &QAbstractItemView::activated,
@@ -345,7 +346,7 @@ GenericProposalWidget::GenericProposalWidget()
     connect(d->m_completionListView->verticalScrollBar(), &QAbstractSlider::sliderReleased,
             this, &GenericProposalWidget::turnOnAutoWidth);
 
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    auto layout = new QVBoxLayout(this);
     layout->setMargin(0);
     layout->addWidget(d->m_completionListView);
 
@@ -357,7 +358,6 @@ GenericProposalWidget::GenericProposalWidget()
 
 GenericProposalWidget::~GenericProposalWidget()
 {
-    delete d->m_model;
     delete d;
 }
 
@@ -384,10 +384,9 @@ void GenericProposalWidget::setUnderlyingWidget(const QWidget *underlyingWidget)
     d->m_underlyingWidget = underlyingWidget;
 }
 
-void GenericProposalWidget::setModel(IAssistProposalModel *model)
+void GenericProposalWidget::setModel(ProposalModelPtr model)
 {
-    delete d->m_model;
-    d->m_model = static_cast<GenericProposalModel *>(model);
+    d->m_model = model.staticCast<GenericProposalModel>();
     d->m_completionListView->setModel(new ModelAdapter(d->m_model, d->m_completionListView));
 
     connect(d->m_completionListView->selectionModel(), &QItemSelectionModel::currentChanged,
@@ -407,7 +406,8 @@ void GenericProposalWidget::setIsSynchronized(bool isSync)
 void GenericProposalWidget::showProposal(const QString &prefix)
 {
     ensurePolished();
-    d->m_model->removeDuplicates();
+    if (d->m_model->containsDuplicates())
+        d->m_model->removeDuplicates();
     if (!updateAndCheck(prefix))
         return;
     show();
@@ -448,12 +448,12 @@ bool GenericProposalWidget::updateAndCheck(const QString &prefix)
                 d->m_model->persistentId(d->m_completionListView->currentIndex().row());
 
     // Filter, sort, etc.
-    d->m_model->reset();
-    if (!prefix.isEmpty())
-        d->m_model->filter(prefix);
-    if (d->m_model->size() == 0
-            || (!d->m_model->keepPerfectMatch(d->m_reason)
-                && isPerfectMatch(prefix, d->m_model))) {
+    if (!d->m_model->isPrefiltered(prefix)) {
+        d->m_model->reset();
+        if (!prefix.isEmpty())
+            d->m_model->filter(prefix);
+    }
+    if (!d->m_model->hasItemsToPropose(prefix, d->m_reason)) {
         d->m_completionListView->reset();
         abort();
         return false;
@@ -550,7 +550,7 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
             d->m_infoFrame->close();
         return true;
     } else if (e->type() == QEvent::ShortcutOverride) {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(e);
+        auto ke = static_cast<QKeyEvent *>(e);
         switch (ke->key()) {
         case Qt::Key_N:
         case Qt::Key_P:
@@ -560,7 +560,7 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
             }
         }
     } else if (e->type() == QEvent::KeyPress) {
-        QKeyEvent *ke = static_cast<QKeyEvent *>(e);
+        auto ke = static_cast<QKeyEvent *>(e);
         switch (ke->key()) {
         case Qt::Key_Escape:
             abort();
@@ -620,14 +620,14 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
 
         default:
             // Only forward keys that insert text and refine the completion.
-            if (ke->text().isEmpty())
+            if (ke->text().isEmpty() && !(ke == QKeySequence::Paste))
                 return true;
             break;
         }
 
         if (ke->text().length() == 1
                 && d->m_completionListView->currentIndex().isValid()
-                && qApp->focusWidget() == o) {
+                && QApplication::focusWidget() == o) {
             const QChar &typedChar = ke->text().at(0);
             AssistProposalItemInterface *item =
                 d->m_model->proposalItem(d->m_completionListView->currentIndex().row());
@@ -658,7 +658,7 @@ bool GenericProposalWidget::activateCurrentProposalItem()
     return false;
 }
 
-GenericProposalModel *GenericProposalWidget::model()
+GenericProposalModelPtr GenericProposalWidget::model()
 {
     return d->m_model;
 }
