@@ -1657,6 +1657,15 @@ TEST(SmallString, StringViewPlusOperator)
     ASSERT_THAT(result, "text and more text");
 }
 
+TEST(SmallString, StringViewPlusOperatorReverseOrder)
+{
+    SmallStringView text = " and more text";
+
+    auto result = "text" + text;
+
+    ASSERT_THAT(result, "text and more text");
+}
+
 TEST(SmallString, StringPlusOperator)
 {
     SmallString text = "text";
@@ -1666,6 +1675,15 @@ TEST(SmallString, StringPlusOperator)
     ASSERT_THAT(result, "text and more text");
 }
 
+TEST(SmallString, StringPlusOperatorReverseOrder)
+{
+    SmallString text = " and more text";
+
+    auto result = "text" + text;
+
+    ASSERT_THAT(result, "text and more text");
+}
+
 TEST(SmallString, ShortStringCapacity)
 {
     ASSERT_THAT(SmallString().shortStringCapacity(), 30);
